/* HDController.h		by Paul Kunz	January 1993
 * Application delegate to make Hippo.bundle into HippoDraw application.
 * Also serves as example on how to incorporate Hippo into an application.
 *
 * Copyright (C)  1993  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */
 
#define HDCONTROLLER_H_ID "$Id: HDController.h,v 1.10 1993/08/04 21:54:42 pfkeb Exp $"

#import "objc/Object.h"

@class Application, MenuCell;
@class HDrawApp;
@interface HDController : Object
{
    HDrawApp	*hippoDraw;
}


- init;
 /* 
  * For the HippoDraw application, we load the Hippo.bundle. For your
  * application you could postpone the initialization until later.
  */
  
/* Application delegate methods that can be passed to Hippo */
- appDidInit:(Application *)sender;
 /*
  * For the HippoDraw application, we pass this on to HDrawApp, 
  * which in turn passes it to DrawApp instance.   It initializes
  * the Tools panel, brings it front, and parses the command arguments,
  * if any, to open a file.
  *
  * For your application, you'll need to send appDidInit to HDrawApp
  * at some point to get correct behaviour of the Tools panel.
  */
  
- (BOOL)appAcceptsAnotherFile:(Application *)sender;
 /*
  * Responds YES so that one can receive name of files from Workspace.
  */
  
- (int)app:sender openFile:(const char *)path type:(const char *)type;
 /*
  * This method is invoked by Application when a file typed owned
  * by the application is double-clicked upon in the workspace.
  * For the HippoDraw application, this is passed on to HDrawApp.
  * For other applications, you should pass this on if the
  * type is ".hdraw" or ".hippo".
  */
  
- app:sender powerOffIn:(int)ms andSave:(int)andSave;
 /*
  * Give the user a chance to save documents.
  */
- (BOOL)menuItemUpdate:(MenuCell *)menuCell;
 /*
  * Sent by application to validate menu items.
  */
  
- (BOOL)validateCommand:(MenuCell *)menuCell;
 /*
  * validates menuCell.   This method is invoked automatically
  * by the updating methods.  Any menu cells that can become
  * disabled and whose target is this class must be validated.
  * The message is passed on to HDrawApp and DrawApp.
  */
  
/* Targets for MenuCells */
- info:sender;
 /*
  * Brings front the HippoDraw info panel.
  */
  
- legal:sender;
 /*
  * Brings front the HippoDraw legal panel with DoE boiler plate.
  */
  
- cernPanel:sender;
 /*
  * Brings front the CERN copyright panel
  */
- new:sender;
 /*
  * Creates a new HippoDraw document. For HippoDraw application
  * it is connected to the Document/New menu cell
  */
 
- open:sender;
 /*
  * Opens a HippoDraw document.
  */
  
- openTuple:sender;
  /* 
   * If main window is a DrawDocument, then passes this message is
   * passed to it to open tuple binary file.   If main window is not
   * a DrawDocument, then one is created and message is passed to it.
   */
   
- openTupleAsText:sender;
  /*
   * Same as openTuple: except used for plain text tuple files.
   */

- closeTuple:sender;
  /*
   * Close currently selected n-tuple..
   */

- setCurrentGraphic:sender;
 /*
  * The "Select" menu cell sends this message to force the
  * draw tool to be the selection tool.
  */
  
- terminate:sender;
 /*
  * Passes this message on to HDrawApp instance to gracefully
  * shut down the application.   HippoDraw checks for any open
  * and modified drawing documents.
  */
- orderFrontTools:sender;
 /*
  * Orders front the graphic tools panel.   
  */
  
- orderFrontInspector:sender;
 /*
  * Orders front the HippoDraw inspector
  */
   
- showTupleAsTable:sender;
 /*
  * Brings front the tuple table
  */
  

- showHelpPanel:sender;
 /*
  * Makes the NXHelpPanel key window and orders it front.
  */
  
- addToAvailFuncs:sender;
 /*
  * For the HippoDraw application, this method is invoked by menu 
  * cell for adding a plot function to list in the Plot Function
  * inspector.
  */
  

/* Following methods are to support interfacing with Mail.app to 
 * send bug reports and suggestions.
 */
- mailSpeaker;
 /* 
  * Creates and returns a Speaker object to talk with MailSendDemo listener.
  * Launches Mail.app if necessary.   Returns nil if not successful.
  */
- sendBug:sender;
 /*
  * Send a bug report
  */
- sendSuggestion:sender;
 /*
  * Send a suggestion.
  */
@end



