@interface DrawDocument : ChangeManager
{
    GraphicView *view;		/* the document's GraphicView */
    Window *window;		/* the window the GraphicView is in */
    PrintInfo *printInfo;	/* the print information for the GraphicView */
    char *name;			/* the name of the document */
    char *directory;		/* the directory it is in */
    char *iconPathList;		/* list of files last dragged over document */
    BOOL haveSavedDocument;	/* whether document has associated disk file */
    UndoText *undoFieldEditor;
    NXDataLinkManager *linkManager;	/* manager of data links */
    
    id	drawInstance;		/* Only instance of global controller */
}

/* Very private instance method needed by factory methods */

- (BOOL)loadDocument:(NXStream *)stream frameSize:(NXRect *)frame frameString:(char *)frameString;

/* Factory methods */

+ (NXZone *)newZone;

+ new:viewClass;
 /*
  * Creates a new DrawDocument and a view from class viewClass
  */

+ newFromStream:(NXStream *)stream;
+ newFromFile:(const char *)file andDisplay:(BOOL)display;
+ newFromFile:(const char *)file;

/* Public methods */

- init;
- free;
- close;	/* Frees (delayed). */

/* Data link methods */

- setLinkManager:(NXDataLinkManager *)aLinkManager;
- showSelection:(NXSelection *)selection;
- copyToPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection cheapCopyAllowed:(BOOL)flag;
- pasteFromPasteboard:(Pasteboard *)pasteboard at:(NXSelection *)selection;
- importFile:(const char *)filename at:(NXSelection *)selection;
- windowForSelection:(NXSelection *)selection;

/* Overridden from ChangeManager */

- changeWasDone;
- changeWasUndone;
- changeWasRedone;
- clean:sender;
- dirty:sender;

/* Public Methods */

- resetScrollers;
- view;
- printInfo;

/* Target/Action methods */

- changeLayout:sender;
- changeGrid:sender;
- save:sender;
- saveAs:sender;
- saveTo:sender;
- saveWindowTo:sender;
- revertToSaved:sender;
- showTextRuler:sender;
- hideRuler:sender;

/* Private method used by saveTo: method */

- changeSaveType:sender;

/* Document name and file handling methods */

- (const char *)filename;
- (const char *)directory;
- (const char *)name;
- setName:(const char *)name andDirectory:(const char *)directory;
- (BOOL)setName:(const char *)name;
- setTemporaryTitle:(const char *)title;
- saveTo:(const char *)type using:(SEL)streamWriter;
- save;
- (BOOL)isSameAs:(const char *)filename;

/* Services menu methods */

- registerForServicesMenu;
- validRequestorForSendType:(NXAtom)sendType andReturnType:(NXAtom)returnType;
- writeSelectionToPasteboard:pboard types:(NXAtom *)types;

/* Window delegate methods */

- windowWillClose:sender cancellable:(BOOL)flag;

- windowWillClose:(Window *)sender;
- windowDidBecomeMain:(Window *)sender;
- windowWillResize:(Window *)sender toSize:(NXSize *)size;
- windowWillMiniaturize:(Window *)sender toMiniwindow:counterpart;
- windowWillReturnFieldEditor:(Window *)sender toObject:client;

/* Menu command validation method */

- (BOOL)validateCommand:(MenuCell *)menuCell;

/* Cursor setting */

- resetCursor;

/*
 * Methods added by HippoDraw
 */
 
- getPageFrame:(NXRect *)frame;
 /*
  * Returns the frame according to the current page layout
  */
@end

@interface DrawDocument(Hippo)

- startArchivingTo:(const char *)directory;
 /*
  * Called immediately before archiving the graphic view with
  * the directory the document will be saved in.
  */
- (int )finishUnarchivingFrom:(const char *)directory;
 /*
  * Called immediately after unarchiving a new document.   Gives
  * the graphic view the path to the directory, so it can 
  * finish unarchiving the items not in the typed stream.
  */
- (BOOL) hasPFunctions;
- startUnarchivingFrom:(const char *)directory;
 /*
  * Loads Minuit and the standard function set.   Should be called
  * before attempting to unarchive the GraphicView if Plot Functions
  * are in the view.
  */
- saveAsExportFile:(const char *)filename;
 /*
  * Saves the hippo displays, and thus ntuples, as native hippo file.
  */

@end
