@interface TextGraphic : Graphic
{
    char *data;			/* the rich text */
    int length;			/* the length of data */
    Font *font;
    View *editView;
    GraphicView *graphicView;
    Text *fe;			/* the field editor text object      */
    				/* used for editing between edit:in: */
				/* and textDidEnd:endChar:           */
    NXDataLink *link;
    char *name;
}

/* Factory methods overridden from superclass */

+ (BOOL)isEditable;
+ cursor;

/* Initialization method */

+ (BOOL)canInitFromPasteboard:(Pasteboard *)pboard;

- init;
- initFormEntry:(const char *)name localizable:(BOOL)isLocalizable;

- initFromStream:(NXStream *)stream;
- initFromPasteboard:(Pasteboard *)pboard;

- (NXRect)reinitFromPasteboard:(Pasteboard *)pboard;
- (NXRect)reinitFromStream:(NXStream *)stream;

/* Link methods */

- setLink:(NXDataLink *)aLink;
- (NXDataLink *)link;

/* Instance methods overridden from superclass */

- (const char *)title;
- (BOOL)create:(NXEvent *)event in:(GraphicView *)view;
- (BOOL)edit:(NXEvent *)event in:(View *)view;
- draw;
- performTextMethod:(SEL)aSelector with:(void *)anArgument;
- changeFont:sender;
- (Font *)font;
- (BOOL)isOpaque;
- (BOOL)isValid;
- (BOOL)isFormEntry;
- setFormEntry:(int)flag;
- (BOOL)writeFormEntryToStream:(NXStream *)stream;
- (NXColor)lineColor;
- (NXColor)fillColor;
- (NXCoord)baseline;
- moveBaselineTo:(NXCoord *)y;

/* Public methods */

- prepareFieldEditor;
- resignFieldEditor;
- (BOOL)isEmpty;
- (char *)data;
- setData:(char *)newData;
- (int)length;
- setLength:(int)newLength;
- setFont:newFont;

/* Text delegate methods */

- textDidResize:textObject oldBounds:(const NXRect *)oldBounds invalid:(NXRect *)invalidRect;
- textDidEnd:textObject endChar:(unsigned short)endChar;

/* Archiving methods */

- awake;
- read:(NXTypedStream *)stream;
- write:(NXTypedStream *)stream;

@end
