/* HDrawApp	By Paul Kunz	May 1992
 * Global object for application incorporating HippoDraw.
 *
 * Copyright (C)  1992  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */
 
#define HDRAWAPP_H_ID "$Id: HDrawApp.h,v 2.34 1993/07/27 19:31:03 pfkeb Exp $"
#import <mach/boolean.h>  /* to get types defined before Hippo's rpc/types */
#import "hippo.h"

#import "objc/Object.h"

@class Application, DrawApp, MenuCell, Panel, Window;
@interface HDrawApp : Object
{
    DrawApp	*drawInstance;
    Panel	*infoPanel;	
    id theNewInspector;		/* the Panel holder for inspectors */
    id legalPanel;		/* the Legal... panel */
    id cernPanel;		/* the CERN copyright panel */
    id inspectTuple;		/* the shared Tuple inspector */
    id inspectPlot;		/* the shared Plot inspector and panel */
    id inspectAxes;		/* the shared Axes inspector and View */
    id inspectStat;		/* the shared Stat inspector and View */
    id inspectCut;		/* the shared Cut inspector and panel */
    id inspectFunc;		/* the shared Function inspector and panel */
    id inspectPFunc;		/* Plot function inspector */
    id ssmanager;		/* SpreadSheetMgr object */
    BOOL	minuitLoaded;	/* flag indicating status of Minuit.bundle */
}

/* Public methods */

+ allocFromZone:(NXZone *)aZone;
 /*
  * Over-rides super class method in order to alloc from
  * newly create NXZone named "HippoDraw".
  */

+ alloc;
 /*
  * Over-rides super class method in order to alloc from
  * newly create NXZone named "HippoDraw".
  */

- addTuple:(ntuple) nt;
 /*
  * Create an HTuple object to handle the ntuple nt, add the handler
  * to the list of handlers of open ntuples, and return HTuple id
  * to the caller.   This method should be used to announce an
  * externally created ntuple to the HippoDraw package.
  */ 
  
- init;

- appDidInit:(Application *)sender;
 /*
  * Responds to application initialization method
  */
  
- loadMinuit:sender;
 /*
  * Dynamically load Minuit.bundle
  */
  
- addToAvailFuncs:sender;
 /*
  * Responds to menu cell by passing message to the Plot Function inspector.
  */
  
- (BOOL)validateCommand:(MenuCell *)menuCell;
 /*
  * validates menuCell.   This method is invoked automatically
  * by the updating methods.
  */
  
- info:sender;
 /*
  * Orders Front the Hippo Info panel
  */
  
- legal:sender;
 /*
  * Load the DoE boiler plate panel.
  */
  
- cernPanel:sender;
 /*
  * Load and display the CERN boiler plate panel.
  */
  
- open:sender;
 /*
  * Opens a HippoDraw document.
  */
  
- terminate:sender;
 /*
  * Responds to Quit MenuCell.
  */
- setCurrentGraphic:sender;
 /*
  * Responds to Select Menu Cell
  */
- orderFrontTupleInsp:sender;
  /* Brings the Tuple inspector panel to view in NewInspector panel
   */
   
- orderFrontTools:sender;
 /*
  * Displays the graphic tools panel.   
  */
  

- (Window *)documentWindow;
 /*
  * Returns the Window of the current Document if any.
  */
  
//- help:sender;
- addPlotOfType:(graphtype_t) type;
 /*
  * Creates and adds a new Plot of specified type to the current document.
  */

- replaceNtuple:(ntuple)oldnt with:(ntuple)newnt andFree:(BOOL)flag;
 /*
  * Replaces ntuple oldnt with newnt and free the old one if flag is YES
  */
- openTuple:sender;
  /* If main window is a DrawDocument, then passes this message is
     passed to it to open tuple binary file.   If main window is not
     a DrawDocument, then is is created and message is passed to it.
   */
- openTupleAsText:sender;
  /* Same as openTuple: except used for plain text tuple files.
   */

- openTuple:pasteBoard userData:(const char *)args error:(char **)errorMsg;
 /*
  * Services method for reading pasteboard as ascii n-tuple.
  */
- closeTuple:sender;
 /*
  * Closes the currently selected n-tuple file.
  */
  
- newDocument;
 /*
  * Creates a new drawing document (and its window) and return it.
  */
  
- showTupleAsTable:sender;
 /*
  * Brings front the tuple table
  */
  
/* Methods over-ridden */

- new:sender;
 /*
  * Creates a new document--called by pressing New in the Document menu.
  * Differs from Draw in that it turns on the grid.
  */
 
- (int)app:sender openFile:(const char *)path type:(const char *)type;
- app:sender powerOffIn:(int)ms andSave:(int)andSave;
 /*
  * Give the user a chance to save documents.
  */
  
- (BOOL)menuItemUpdate:(MenuCell *)menuCell;
 /*
  * Sent by application to validate menu items.
  */
  
- (int)appOpenHippoFile:(const char *)path;
  /*
   * Internal method to open .hippo file
   */
/* Public methods internal to Hippo.bproj */
- newInspector;
- currentInspector;
  /*
   * return the current forward inspector
   */
   
- inspectTuple;
  /* Returns the shared Tuple Inspector object
   */
   
- inspectPlot;

- inspectFunc;
- inspectPFunc;
- inspectCut;
- inspectAxes;
@end



