/* InspectCut.h		by Mike Gravina		November 1991
 * Controls the cuts on plots
 *
 * Copyright (C)  1991  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */ 

#define INSPECTCUT_H_ID "$Id: InspectCut.h,v 2.15.2.2 1994/02/08 20:29:16 rensing Exp $"

#import "InspectBase.h"
#import "Plot.h"


@interface InspectCut:InspectBase
{
 /* GUI objects */
    id		hiSlider;
    id		loSlider;
    id		cutValueForm;
    id		zpSwitch;	/* Zoom/Pan mode switch Button */
    id		tpBrowser;	/* NXBrowser showing tuple data titles */
    id		tpBrowserCells;
    id		cutBrowser;	/* NXBrowser showing cuts added */
    id		cutBrowserCells;
    id		showCutsRadio;	/* showAllState control button */
    id		buttonMatrix;	/* Add/Remove buttons */
/* other objects */
    List	*tupleCutList;	/* List of cuts with current ntuple */
    List	*cutList;	/* List of cut Plots */
    Plot	*cutPlot;	/* current cut Plot */
    
    id	hTuple;
    float cutLow, cutHigh;		/* current limits of cut */
}
- initInspFor:aDraw;
 /*
  * creates a new InspectCut for global controler aDraw.
  * After initialization, adds itself to the NewInspector.
  *
  * This method is the designated initializer for the class.
  */
  
- newValueInSlider1:sender;
	/* action when slider 1 is changed */
	
- newValueInSlider2:sender;
	/* action when slider 2 is changed */
	
- newValueInForm:sender;
	/* action when a new value 1 or value2 is typed in as text */

- newCut:sender;
 /*
  * Creates a new cut Plot and applies it to the currently selected
  * Plots.
  */

- add:sender;
 /*
  * Adds existing selected cut to currently selected Plots
  */
  
- invert:sender;
 /*
  * Inverts the meaning of the cuts on selected plots
  */
  
- remove:sender;
 /*
  * Removes the cut Plot from selected Plot(s) and other plot that
  * are cutting selected Plot(s) if they also depend on the cut Plot
  */
  
- replace:sender;
 /*
  * Responds to replace button by replacing the selected Cut with a new
  * one.
  */
/*- deleteCut:plot;*/
 /*
  * Delete cut plot from the view's cut plot list and updates
  * panel view.
  */
  
/*- addCut:plot;*/
 /*
  * Adds the cut plot to the view's cut plot list and updates
  * panel view.
  */
  
- showCutsRadio:sender;
 /*
  * Changes the cut Browser between showing cuts of selected plot
  * or showing all cuts
  */
   
- mouseMoved:(const NXPoint *)offset in:sender withKey:(int)flag;
 /* used for trapping mouse movements.
  */


- updateView;
 /*
  * Updates the Inspector's View when it first comes up,
  * or when there is a change
  */
  
- selectCut:sender;
 /*
  * Responds to selection in the cut Browser.
  */

- (int) browser: sender fillMatrix: matrix inColumn: (int) column;
 /* delegate method of NXBrowser.   Will modify matrix and fill it with
  * tuple data 
  */

@end
