/* InspectTuple	by Paul Kunz	October 1991
 * The supervisor for the tuple inspection panel
 * 
 * Copyright (C)  1991  The Board of Trustees of
 * The Leland Stanford Junior University.  All Rights Reserved.
 */ 

#define INSPECTTUPLE_H_ID "$Id: InspectTuple.h,v 2.11.2.1 1993/10/25 18:13:57 pfkeb Exp $"

#import "InspectBase.h"

#define HD_OK 0
#define HD_NOTEXIST -2
#define HD_CANCEL -1
#define HD_REPLACE 1

@class HTuple;

@interface InspectTuple:InspectBase
{
    id		tupleBrowser;	/* NXBrowser showing tuple titles */
    id  	colBrowser;	/* NXBrowser showing tuple data titles */
    id  	infoForm;	/* Form to hold current axis labels */
    id  	tpAxisButtons;	/* Matrix of Buttons to select axis */
    id		graphTypMatrix;	/* Matrix to select graph type */
    id		popUpButton;	/* Button holding PopUpList */
    id		accessoryView;	/* accessory View for OpenPanel */
    id		msgMatrix;	/* Matrix of TextFields for message */
    id		copyButton;	/* Button to make copy on save */
    List	*tupleList;	/* List for opened tuples */
    List	*tmpTupleList;	/* List used temporary for re-opening file */
    OpenPanel	*openPanel;	/* global OpenPanel object */
    ntuple	selectedTuple;	/* The last selected tuple */
    graphtype_t graphtype;
    int 	tpAxisIndex[5];	/* index into tuple for each axis */
    int		popUpRow;	/* row selected by PopUpList */
}
- addPlot:sender;
 /*
  * Adds a new plot to the drawing document.
  */
  
- axisMatrixClick:sender;
  /*
   * Responds to change in selected axis by restoring selected cell
   * in tuple Browser 
   */
   
- colBrowserSingleClicked:sender;
 /*
  * Does nothing, but keeps the Matrix happy.
  */
- colBrowserDoubleClicked:sender;
 /*
  * Responds to double click in column browser by changing attribute
  * of currently selected plot(s)
  */
- graphOption:sender;
 /*
  * Responds to click in graph type options.
  */
  
- addTuple:(ntuple) nt;
 /*
  * Creates a new HTuple object as a handler for the externally created
  * ntuple nt, add it to the list of handlers and returns the id of it.
  */
  
- addPlotOfType:(graphtype_t) plotType;
 /*
  * Creates and returns a plot of specified type.
  */
  
- initInspFor:aDraw;
 /*
  * Initalizes an object for the global controler aDraw.
  * And adds itself to the NewInspector list.
  */
  
- openTuple:sender;
  /* Opens new binary tuple
   */
   
- openTupleAsText:sender;
  /* Opens new text tuple
   */
   
- formChange:sender;
 /* 
  * Responds to change in selected row of popUpList.
  */
  
- (int) openTupleFile:(const char *)filename by:(BOOL) refFlag;
  /*
   * Opens new tuples from file filename.
   * The by reference flag for the tuple is set by value of refFlag.
   * Checks if tuple with the same filename already is opened.
   * Returns HD_OK on success, HD_CANCEL if user canceled, or
   * HD_REPLACE if user wants to replace open tuple with one in file.
   */
   
- openTuple:pasteBoard
    userData:(const char *)args
    error:(char **)errorMsg;
  /*
   * Opens an ascii n-tuple from the pasteboard
   */
   
- saveAsExportFile:(const char *)filename;
 /*
  * Saves the Hippo displays and thus ntuple only
  */
- newTuple:(ntuple) nt;
  /* Adds a new tuple to the tupleList List object.  
   * Do not use this method, use addTuple: instead.
   */
   
- hTupleForFile:(const char *)filename index:(int)intValue;
  /*
   * Returns the HTuple object that manages the intValue-th ntuple in
   * file filename, or nil if none found.
   */

- tupleList;
  /*
   * Returns the complete list of open tuples
   */
   
- tupleListForFile:(const char *) filename;
  /*
   * returns a List object containing HTuple objects whose
   * n-tuple is from filename.
   */
   
- setTuple:(ntuple) atuple;
  	/* Sets the tuple with already prepared display held by this object
   	*/

- (ntuple) currentTuple;
  /* Returns the currently selected tuple
   */
   
- currentHTuple;
 /*
  * Returns the currently selected HTuple object or
  * nil if none is selected.
  */
  
- bindDisplays;
 /*
  * Sends bindDisplays message to each DrawDocument View.   Used
  * when replacing a n-tuple file.
  */
  
- updateView;
 /*
  * Updates the contents of the Inspector's View
  */
  
- updateEmptySelection;
 /*
  * Updates its view for empty selection by enabling all the
  * axis selection buttons.
  */
  
- (int) addTuplesIfAbsent:newList;
 /*
  * Adds the HTuples contain in the List newList to the tuple List
  * maintained by the receiving object if they are absent.  
  * Frees HTuples in newList
  */
  
- tupleBrowserClick:sender;
  /*
   * Responds to click in tuple browser by sending replace
   * tuple message to the graphics view
   */

- replaceNtuple:(ntuple)oldnt with:(ntuple)newnt andFree:(BOOL)flag;
  /*
   * Replaces ntuple oldnt with newnt and frees old if flag is YES.
   * Searchs all for all open documents.
   */
   
- closeTuple:sender;
  /*
   * Responds to the Close Button by closing the n-tuple after
   * informing all DrawDocument Views.
   */
   
- updateColBrowser;
 /*
  * Does a reload on the tuple column browser
  */
  
- changeImportMode:sender;
 /*
  * Changes the Import by Reference mode to the opposite that it is for
  * the currently selected HTuple.
  */
  
- setInspectorFor:(id)plot;

/* Delegate Methods for NXBrowsers */
- (BOOL)browser:sender selectCell:(const char *)title inColumn:(int)column;
- (int) browser: sender fillMatrix: matrix inColumn: (int) column;
  /* delegate method for NXBrowser that displays titles of active tuples
   */
- (int) fillTupleMatrix:matrix;
  /* Fills the tuple matrix with tuple names 
   */
- (int) fillColumnMatrix:matrix;
  /* Fills the column matrix with column names
   */
- setMsgMatrix:anObject;
  /*
   * Receives the id of the message matrix and clears the text
   * set by the Interface Builder
   */
- startUnarchivingFrom:(const char *)directory;
 /*
  * Not implemented yet
  */
@end
