/*
 * hippoplotps.c - Postscript routines for displaying hippo ntuples.
 *
 * Copyright (C)  1991  The Board of Trustees of The Leland Stanford
 * Junior University.  All Rights Reserved.
 *
 * $Header: /nfs/ebnextk/LocalSources/hippoplotamus/hplotNxt.h,v 5.0.2.1 1993/12/15 22:46:10 mfg Exp $
 *
 */

#include <dpsclient/wraps.h>
#include "hpsWraps.h"

#define _HPLOTNXT_H_RCSID_ "$Id: hplotNxt.h,v 5.0.2.1 1993/12/15 22:46:10 mfg Exp $"

/*
 * initPlot - specify the outer and inner rectangles in device coords,
 * and the inner rectangle again in user coords. This sets global
 * values that are used by the other routines, so it should be called
 * whenever any rectangle specification is changed.
 */
int initPlot_NeXT(rectangle *rect1, rectangle *rect2, rectangle *rect3);

/*
 * Draw lines between coordinates
 */
int drawLine_NeXT(float *coords, int nCoords, linestyle_t ls);

/*
 * Draw a series of points, using the symbol specified
 */
int drawPoints_NeXT(float *coords, int nCoords, plotsymbol_t symbol,
		    float symbolSize);

/*
 * Draw vertical error bars.  We use only the x part of the
 * coordinates, plus the y-low and y-high pairs.
 */
int drawYError_NeXT(float *coords, float *errorYs, int nCoords);

/*
 * Draw horizontal error bars.  We use only the y part of the
 * coordinates, plus the x-low and x-high pairs.
 */
int drawXError_NeXT(float *coords, float *errorXs, int nCoords);

/*
 * Place text at a given location in device coords. X alignment is
 * specified as 'L', 'R', or 'C' (or lower case), meaning that the
 * left, right, or center of the text is positioned at the xy
 * location.  Similarly Y alignment is given as 'B', 'C', or 'T', for
 * bottom, center or top.  Rotation is then performed, given in
 * degrees anti-clockwise.  The variable 'fontSize' is the size in
 * 'points'.
 */
int drawText_NeXT(char *string, float x, float y, float fontSize, 
		  float rotation, char xAlign, char yAlign);

/*
 * draw ticks and labels along Y axis
 */
int drawYTicks_NeXT( float *ticks, int nticks, float tickwidth, int side );
int drawXTicks_NeXT( float *ticks, int nticks, float tickwidth, int side );

/*
* shade portions of plot to indicate areas removed by a cut
*/
int shade_NeXT(float xlow, float xhigh, float ylow, float yhigh );


/*
 * plot "x10^mag" at specified location
 */
int drawMag_NeXT( float x, float y, int mag, float fontSize );

/*
 * greyscale/color 2D plot for the NeXT
 */
int drawColor2D_NeXT(int nXBins, int nYBins, float binMin, float binMax, 
		     float *data, int useColor );


int drawLego2D_NeXT(display disp);

int drawScatter3D_NeXT(display disp, float *coords, int nCoords);

