#ifndef _hippoplotPS_h_
#define _hippoplotPS_h_

/*
 * hippoplotPS.c - Postscript routines for displaying hippo ntuples.
 *
 * Copyright (C)  1991  The Board of Trustees of The Leland Stanford
 * Junior University.  All Rights Reserved.
 *
 * $Id: hplotPS.h,v 5.0 1993/08/17 21:55:57 rensing Exp $
 *
 */

#include "hippo.h"

#define _HIPPOPLOTPS_H_RCSID_ "$Id: hplotPS.h,v 5.0 1993/08/17 21:55:57 rensing Exp $"

/*
 * Initialize the plot driver. Specify that the given file pointer will
 * be used for output. This routine should be called once for a new file.
 * If the user wants to change files, she should close the current file,
 * open a new one, and call h_setPlotDrvr again.
 */
int initDrvr_PS( FILE *fl );

/*
 * endDrvr - do any cleanup work when you are done with this driver.
 */
int endDrvr_PS( void );


/*
 * initPlot - specify the outer and inner rectangles in device coords,
 * and the inner rectangle again in user coords. This sets global
 * values that are used by the other routines, so it should be called
 * whenever any rectangle specification is changed.
 */
int initPlot_PS(rectangle *rect1, rectangle *rect2, rectangle *rect3);

/*
 * endPage - end the page.
 */
int endPage_PS( void );

/*
 * Draw lines between coordinates
 */
int drawLine_PS(float *coords, int nCoords, linestyle_t ls);

/*
 * Draw a series of points, using the symbol specified
 */
int drawPoints_PS(float *coords, int nCoords, plotsymbol_t symbol,
		    float symbolSize);

/*
 * Draw vertical error bars.  We use only the x part of the
 * coordinates, plus the y-low and y-high pairs.
 */
int drawYError_PS(float *coords, float *errorYs, int nCoords);

/*
 * Draw horizontal error bars.  We use only the y part of the
 * coordinates, plus the x-low and x-high pairs.
 */
int drawXError_PS(float *coords, float *errorXs, int nCoords);

/*
 * Place text at a given location in device coords. X alignment is
 * specified as 'L', 'R', or 'C' (or lower case), meaning that the
 * left, right, or center of the text is positioned at the xy
 * location.  Similarly Y alignment is given as 'B', 'C', or 'T', for
 * bottom, center or top.  Rotation is then performed, given in
 * degrees anti-clockwise.  The variable 'fontSize' is the size in
 * 'points'.
 */
int drawText_PS(char *string, float x, float y, float fontSize, 
		  float rotation, char xAlign, char yAlign);

/*
 * draw ticks and labels along Y axis
 */
int drawYTicks_PS( float *ticks, int nticks, float tickwidth, int side );
int drawXTicks_PS( float *ticks, int nticks, float tickwidth, int side );

/*
* shade portions of plot to indicate areas removed by a cut
*/
int shade_PS(float xlow, float xhigh, float ylow, float yhigh );


/*
 * plot "x10^mag" at specified location
 */
int drawMag_PS( float x, float y, int mag, float fontSize );

/*
 * greyscale/color 2D plot
 */
int drawColor2D_PS(int nXBins, int nYBins, float binMin, float binMax, 
		     float *data, int useColor );



int drawLego2D_PS(display disp);

int drawScatter3D_PS(display disp, float *coords, int nCoords);

#endif				/* ifndef _hippoplotPS_h_ */
