
#import "Histogram.h"
#import "DataCounter.h"
#import "HistogramView.h"
#import "ToolAndInspectorProtocols.h"

@implementation Histogram

// If the object has an inspector, we'll load it here.  Initialize variables
// that will be displayed by the inspector.
- init
{
	NXBundle *bundle;
	char path[MAXPATHLEN+1], inspectorName[MAXPATHLEN+1];
	NXRect viewBounds = {0, 0, 300, 300};
	
	[super init];

	strcpy(inspectorName, [[self class] name]);
	strcat(inspectorName, "Inspector");
	 
	bundle = [NXBundle bundleForClass:[self class]];

	// If the Inspector nib file exists, load it. 	
		if([bundle getPath:path forResource:inspectorName ofType:"nib"])
			[NXApp loadNibFile:path owner:self withNames:NO 
														fromZone:[self zone]];

	// Initialize variables that will be displayed by the inspector.
		intervalWidth = 1;
		accuracy = 0;
		horizontalAxisLabel = (char *)malloc(1);
		verticalAxisLabel = (char *)malloc(1);
		strcpy(horizontalAxisLabel, "");
		strcpy(verticalAxisLabel, "");
		
	// make dataCounter and histogramView objects 
		dataCounter = [[[bundle classNamed:"DataCounter"] 
											allocFromZone:[self zone]] init];
		histogramView = [[[bundle classNamed:"HistogramView"] 
									allocFromZone:[self zone]] 
													initFrame:&viewBounds];
		
	return self;
}

// methods to be called by the inspector's ok and revert methods

- (float)intervalWidth
{
	return intervalWidth;
}

- (int)accuracy
{
	return accuracy;
}

- (char *)horizontalAxisLabel
{
	return horizontalAxisLabel;
}

- (char *)verticalAxisLabel
{
	return verticalAxisLabel;
}

- setIntervalWidth:(float)width
{
	intervalWidth = width;
	return self;
}

- setAccuracy:(int)numDecPlaces
{
	accuracy = numDecPlaces;
	return self;
}

- setHorizontalAxisLabel:(const char *)label
{
	horizontalAxisLabel = (char*)realloc(horizontalAxisLabel, strlen(label)+1);
	strcpy(horizontalAxisLabel, label);
	return self;
}

- setVerticalAxisLabel:(const char *)label
{
	verticalAxisLabel = (char *)realloc(verticalAxisLabel, strlen(label)+1);
	strcpy(verticalAxisLabel, label);
	return self;
}

// WBTool protocol methods
 
- evaluate:sender
{
	float **data;
	int numCols, numRows, numIntervals, row;
	float *frequencies, *boundaries;
	BOOL nonZero;
	
	// get the data from Improv, return if no data or no good data
		if ([improvConnector floatData:&data fromSelectedColumnsCount:&numCols 
									byRows:NO dataCount:&numRows] != 0)
		{
			NXRunAlertPanel("Alert", "There was some problem in retrieving "
									"the data from Improv.  Make sure that the" 									
									" worksheet containing the data is "
									"on-screen and that the column with the "
									"data is highlighted.", NULL, NULL, NULL);
			return self;
		}
		
		if (numCols != 1)
		{
			NXRunAlertPanel("Alert", "You need to highlight one (and only one)" 
									" column of data in Improv.  Highlight it"
									" by clicking on its label.", 
									NULL, NULL, NULL);
			[improvConnector freeData:data count:numCols];						
			return self;
		}
		
		for (nonZero = NO, row = 0; (row < numRows) && !nonZero; row++)
			if (data[0][row] != 0)
				nonZero = YES;

		if (!nonZero)
		{
			NXRunAlertPanel("Alert", "You need to supply some non-zero data.",
												NULL, NULL, NULL);
			[improvConnector freeData:data count:numCols];						
			return self;
		}
			
	// count the frequencies
		[dataCounter findNumIntervals:&numIntervals boundaries:&boundaries 
					forData:data[0] count:numRows intervalWidth:intervalWidth];

		frequencies = [dataCounter countFrequenciesForData:data[0] 
										count:numRows numIntervals:numIntervals 
										boundaries:boundaries];

	// graph the data
		[outputViewBox insertView:histogramView];

		[histogramView setHorizAxisTitle:horizontalAxisLabel];
		[histogramView setVertAxisTitle:verticalAxisLabel];
		
		[histogramView drawHistogramNumBars:numIntervals 
							boundaries:boundaries accuracy:accuracy 
							frequencies:frequencies];
		[[histogramView window] orderFront:self];

	// free stuff
		free(frequencies);
		free(boundaries);
  		[improvConnector freeData:data count:numCols];
			
	return self;
}

- inspector
{
	return inspector;
}

- (char **)requiredConnectors
{
	static char *connectors[] = {"ImprovConnector", NULL};
	return connectors;
}

- setInputText:text
{
	inputText = text;
	return self;
}

- setOutputText:text
{
	outputText = text;
	return self;
}

- setOutputViewBox:box
{
	outputViewBox = box;
	return self;
}

- setImprovConnector:theImprovConnector
{
	improvConnector = theImprovConnector;
	return self;
}

@end
