/* Mtypes.h - Copyright 1993 Steve Ludtke */
/* This file contains type definitions required by various classes */

#include <3Dkit/3Dkit.h>

#define SQR(x) ((x)*(x))

struct MVatm { unsigned char anum,type; float x,y,z,c; };
struct MVbnd { unsigned short n1,n2; unsigned char type; };

/* SelectView */
#define MAXTEXT 50

struct SELDAT {
	char text[MAXTEXT];	/* text to display for each line */
	short sel;			/* number selected, ie - 1st selected=1 */
	short stab;			/* sort of a separate array containing an ordered */
						/* of the selected cells */
};

/* NrgObj */
/* AMBER TABLES */
struct BONDL { unsigned char at1,at2; float k,r; };
struct NONB { unsigned char an,tp; float r,e; };
struct HBOND { unsigned char at1,at2; float c,d; };

/* MolObj */
#define MAXMOL 2		/* max # of molecules, currently only 1 supported */
#define MAXAA 150		/* max # amino acids/molecule */
#define NEL 86			/* # elements in data file */

typedef struct { float x,y,z; } Vec;	/* vector structure corresponding */
										/* to RtPoint */
/* element information from covalent.dat */
struct ELINFO {
char name[2];
float rad;
float arad;
RtColor color;
id image;
};

/* a bond between atoms */
struct LBOND {
int n1,n2;		/* the 2 atoms which are bonded */
float d,w;		/* currently unused, length and weight of bond */
char type;		/* bond type, not used consistently yet */
unsigned char ambtp;	/* entry # in amber bondl table */
};

/* an atom */
struct ATOM {
unsigned char anum;			/* atomic number */
unsigned char type;			/* additional atom type info, inconsistent usage */
unsigned char ambtp;		/* entry # in amber nonbonded table */
short numb,bnd[4];			/* atoms bonded to this one, max 4 for now */
unsigned short tag:1;
unsigned short sel:1;
unsigned short lab:1;		/* flags used for molecule manipulations */
float charge;				/* point charge for atom (AMBER) */
};

/* atom ellipticity info */
struct ELLIPSE {
	float x,y,z;
	float dz,dx,dz2;
};

/* an amino acid */
struct AMINO {
short n,c1,c2,cn,cn2,h,o,sc,sc2;	/* points to individual atoms in the  */
									/* backbone for dihedral calcs. */
float phi,psi,omega,chi;/* dihedrals, standard IUPAC definition */
short anum;				/* type of amino acid (# in acid[] array) */
char sel;				/* flag used for manipulations */
};

typedef struct ATOM Atom;
typedef struct ELLIPSE Ellipse;

/* molecule structure, currently the program only supports one molecule */
/* at a time */
struct MOLECULE {
Atom *atom;				/* array of atoms in the molecule */
Ellipse *ell;			/* ellipticity info, NULL if not present */
RtPoint *coord;			/* atom coordinates, seperate from atom struct. */
						/* for drawing efficiency */
struct LBOND *lb;		/* table of bonds */
int numa,numlb,numaa;	/* number of atoms,bonds, and amino acids */
RtInt *verts,*nverts;	/* array used for quick drawing */
struct AMINO am[MAXAA];	/* array of amino acids, not dynamically alloc yet */
};

typedef struct MOLECULE Molecule;

/* D3View */
#define TIMESTEP .1		/* Time between updates (in seconds)*/

#define DRC (M_PI/180.0)	/* degree->radian conversion */

/* structure used in quick drawing mode */
struct DSORT {
RtPoint c;
unsigned anum:8;
unsigned atom:22;
unsigned sel:1;
unsigned lab:1;
};

/* 3d point structure (same as Vec defined above) */
typedef struct {
	float x,y,z;
} Point;

/* AminoView */
/* definition of amino acid info structure, currently contains only */
/* 20 of the most common amino acids + unknown */
typedef struct { char *d1,*d2,*d3; long wt; int phob; short nc,ns,no,nn; } ACID;

#define AADEF 21	/* number of acids in info table (see AminoView.m) */
#define MAXSEQ 100	/* max # amino acids in AminoView sequence */

/* wraps.psw - postscript font selection routines */
void PSsethel();
void PSsetohlfs();
void PSsetsymb();
void PSsetsma();

/* Inspector */

struct MODIFY { short nsel,sel[4]; };
#define NMOD 3
