
/* **********************************  SignalView.m ******************************* */

/* Generated by Interface Builder */

#import "signalView.h"				/* header file for this main file */
#import<math.h>					/* header file for math operations */
#import <dsp/dsp.h>				/* header file for acessing DSP functions */
#import <appkit/Form.h>			/* header file for text field objects  */
#import<appkit/Slider.h>			/* header file for slider objects */
#import<dpsclient/wraps.h>		/* header file for using PS and NX graphic functions */
#import <appkit/color.h>			/* header file for using color composing */
#import <appkit/NXColorWell.h>	/* header file for using color well objects */
#import <appkit/Matrix.h>			/* header file for using text filed matrix */
#import <appkit/Application.h>		/* header file for "Application object" */
#import <appkit/NXCursor.h>		/* header file for using cursor operations */
#import <appkit/Window.h>			/* needed for dynamic window drawing */
#import <objc/objc.h>				/* needed for BOOL values YES, NO */
#import <appkit/ButtonCell.h>		/* header file for using buttol cell object */

@implementation signalView

- initFrame:(const NXRect *) frameRect
{
	int i;									/* used as counter */
	printf("Executing  SignalLab \n");		/* prints message on console or terminal */
	[super initFrame:frameRect];			/* initializes signalView object */
	
	crossHairCursor = [NXCursor newFromImage:[NXImage newFromSection:"crossHair.tiff"]];
	/* loads a tiff file " crossHair.tiff" as cursor when mouse is the view of signalView */
	
	hotSpot.x = 8.0;						/* the center point (x) of the 17 x 17 pixel cursor */
	hotSpot.y = 7.0;						/* the center point (y) of the 17 x 17 pixel cursor */
	[crossHairCursor setHotSpot:&hotSpot];	/* origin of cursor is set to the center of image */
	freq1To17Value = 9;					/* default value to generate 100 Hz */
	freqSliderValue = 135;					/* actual value used to generate 100 Hz */
	voltSliderValue = 30.0;					/* default value of amplitude, 3.0 Volts */
	dacSelected = 0x00E;					/* default hex value to select DAC #1 */
	circularSlider1Value = 360;				/* default ending degrees value of sine wave */
	circularSlider2Value = 0;				/* default starting degrees value of sine wave */
	waveSelected = 0;						/* default for no wave generation */
	gridFlag = 1;							/* default to set on grid of "scope" */
	voltString1 = " CH1:1 Volts / Division";	/* default text message displayed on "scope" */
	voltString2 =  "CH2: 1 Volts / Division";	/* default text message displayed on "scope" */
	timeString1 =  "CH1: 2 mSec / Division ";	/* default text message displayed on "scope" */
	timeString2 =  "CH2:2 mSec / Division "; 	/* default text message displayed on "scope" */
	vDiv1Value = 1;						/* default scale value for amplitude on channel  #1 */
										/* 1 Volt/Division  */
	vDiv2Value = 1;						/* default scale value for amplitude on channel  #2 */
										/* 1 Volt/Division */
	freqScale1Factor = 1; 					/* default scale value for frequency on channel  #1 */
										/* 2 mS/Division */
	freqScale2Factor = 1;					/* default scale value for frequency on channel  #2 */
										/* 2 mSec/Division */
	sqSliderValue = 0.5;					/* default duty cycle value ( 50 % ) for pulse */
										/* waveform generation */
	staircaseStep = 0.5;					/* default number of steps ( 5 steps ) for staircase */
										/* waveform generation */
	beamFocusValue = 0.5;				/* default size of "beam" of "scope" */
	PSsetalpha(1.0);						/* default transparency value (opaque) for all colors*/
										/* used on "scope" */
	
	/* Hue Saturation Brightness (HSB) format is ued for color composition */

	backHue = 0.5385; 					/* default background hue value*/
	backSat = 0.9333;  						/* default background saturation value */
	backBrt = 0.3250; 						/* default background brightness value */
	backAlpha = 1;						/* default background transparency value */
	/* background color is set to dark blue */
	
	gridHue = 0.1803; 						/* default grid hue value */
	gridSat = 0.5725;						/* default saturation hue value */
	gridBrt = 0; 							/* default grid brightness value */
	gridAlpha = 1;						/* default grid transparency value */
	/* grid color is set to black */
	
	traceHue = 0.3836; 					/* default trace beam hue value */
	traceSat = 0.9490;						/* default trace beam saturation value */  
	traceBrt = 1;							/* default trace beam brightness value */
	traceAlpha = 1;						/* default trace beam transparency value */
	/* trace beam color is set to bright light blue */
	
	textHue = 0.5409;						/* default text hue value */
	textSat = 0.733;  						/* default text saturation value */
	textBrt = 0.6666; 						/* default text brightness value */
	textAlpha = 1;						/* default text transparency value */
	/* text color is set to dark blue, slightly darker than background color  */
	
	[backColorWellOut setContinuous:YES];	/* continuous update of background color */ 

	[backColorWellOut activate:(int)YES];	/* activates background color well */
		
	[gridColorWellOut setContinuous:YES];	/* continuous update of background color */
 	
	[gridColorWellOut activate:(int)YES];	/* activates grid color Well */
	
	[traceColorWellOut setContinuous:YES];	/* continuous update of trace beam color */ 
	
	[traceColorWellOut activate:(int)YES];	/* activates trace beam color well */
	
	[textColorWellOut setContinuous:YES];	/* continuous update of text color */ 
	
	[textColorWellOut activate:(int)YES];	/* activates text color well */
	
	vpos1Value = ( (bounds.size.height/10) * 7 );
	/* sets vertical position of trace beam , channel 1 to accomodate in the upper portion of the  */
	/* "scope" */
	
	vpos1FineValue = 0;	/* fine setting of vertical position, trace beam channel 1 */
	
	vpos2Value =  ( (bounds.size.height/10) * 3 );
	/* sets vertical position of trace beam , channel 2 to accomodate in the lower portion of the  */
	/* "scope" */
	
	vpos2FineValue = 0;	/* fine setting of vertical position, trace beam channel 2 */
	
	ch1Selected = 1;		/* sets channel 1 input to "off" */ 
	ch2Selected = 1;		/* sets channel 2 input to "off" */
	scrollValue = 0;		/* default origin on "scope ", left portion */
	storeSampleValue = 1;	/* default to store samples data from ADC in "sample1" */
	periodicValue = 1;		/* default value (periodic), for reproducing sampled data */
	xValue = 220;			/* default value to reproduce samples data at X1 frequency */
	
	
	/* The array of buffers used to store lookup data  (dataDSP[]) for the signal generator is filled 	* with zeroes.
	* The array of buffers used to store sampled data from ADC AD7703 (DSPSample1 & 	* DSPSample2 ) is filled with zeroes.
	*/
	i = 0;
	do {
			dataDSP[ i ] = 0;			/* lookup data for signal generator */
			dataDSPSample1[ i ] = 0;	/* sampled data from ADC */
			dataDSPSample2[ i ] = 0;	/* sampled data from ADC */
			i++;
	     } while (i <= 30000);			/* maximum capacity of sampled data */
	
	return self;						/* self is the return value of this method */
}


- drawSelf:(NXRect *)rects :(int) rectCount

/* Invoked when '[self display]' message is sent.  Initially, this method draws the grid, sets the
* "scope" color, beam thickness, intensity and color.  It then responds to all action methods
* which end with [self display], to generate a selected signal of selected amplitude, frequency
* and other parameters.  It also displays the captured data on the "scope"
*/

{
	int  dacCount, counter,  acounter;			/* integer variables */
	float  angle, x2, y2, inc1, inc2, xpos, SQvalue;

	PSsethsbcolor ( backHue,  backSat, backBrt); 
	/* sets background color of the "scope" */
	
	PSsetalpha( backAlpha);				/* sets background transparency of the "scope" */
	NXRectFill (&bounds);				/* fills the backgroung of "scope" to set color */
	PSsethsbcolor ( gridHue,  gridSat, gridBrt);	/* sets grid color of the "scope" */
	
	PSsetalpha( gridAlpha);				/* sets transparency value of grid */
	NXFrameRect(&bounds);				/* the frame of the rectangle is set to grid color */
	PSnewpath();							/* defines a new path for drawing */

	/*------------------------------------------  draws grid on screen ----------------------------------------------*/
	
	if (gridFlag == 1)				/* checks if grid must be turned on */
	{							/* executed if "yes" */
	PSsetlinewidth (1.5);			/* sets grid line width */
	PSsethsbcolor ( gridHue,  gridSat, gridBrt);                       /* sets grid color of the "scope" */
	PSsetalpha( gridAlpha);				/* sets transparency value of grid */
	PSmoveto (0.0, bounds.size.height/2.0);	/* moves to left-center of the rectangle view */
	
	PSlineto(bounds.size.width, bounds.size.height/2);
	/* draws horizontal line to the right edge of the view rectangle */
	
	PSstroke();							/* makes the drawn line visible */ 
	PSnewpath();							/* new path for drawing */
		PSsetlinewidth(0.3);				/* sets drawing line width */
		inc1 = ( bounds.size.height / ( 10  * 5 ) );		/*  divides vertical length of the rectangle into 10 equal divisions */
		
		inc2 = 0;							/* initial increment value */
		counter = 0;						/* initialize counter */
		
		 /*------------ Draws Horizontal Grid Sub division markers-------------  */
		/* executes a loop to to subdivide each division into 5 marks */
		do {								   
			PSmoveto( (bounds.size.width/2 ) - ( inc1/2), inc2);
			PSlineto( ( bounds.size.width/2)  + (inc1/2), inc2);
			inc2 = inc2 + inc1;
			counter++;
		} while (counter <= (10 * 5));
		
		inc1 = ( bounds.size.width / ( 22  * 5 ) );
		/* divides horizontal width of the rectangle into 22 equal divisions */
		
		inc2 = 0;							/* initial increment value */
		counter = 0;						/* initialize counter */
		
		/*------------ Draws Vertical Grid Sub division markers-------------  */
		/* executes a loop to to subdivide each division into 5 marks */
		do 
		{								    			PSmoveto( inc2, (bounds.size.height /2 ) - (inc1/2) );
			PSlineto( inc2, ( bounds.size.height/2)  + (inc1/2) );
			inc2 = inc2 + inc1;
			counter++;
		}	while (counter <= (22 * 5));
		PSstroke();						/* makes the marked lines visible */
 		PSnewpath();						/* new path for drawing */
	PSsetlinewidth(1.5);					/* sets line width */
	
	PSmoveto(bounds.size.width/2, 0);
	PSlineto(bounds.size.width/2, bounds.size.height);
	/* draws a vertical line at the center of the rectangle */
	
	PSstroke();							/* makes the drawn line visible */ 
			
	PSnewpath();							/* new path for drawing */
	PSsetlinewidth(0.3);					/* sets line width */
		PSmoveto( 0, 0);					/* origin of rectangle */
		
		inc1 = ( bounds.size.height / 10 );
		/* increment value set to 1/10th of the rectangle */
		
		inc2 = 0;							/* initial increment value */
		counter = 0;						/* initialize counter */
		
		/* draws ten horizontal lines across the rectangle */ 
		do {								     
			PSlineto(bounds.size.width, inc2);		/* draws lines in increments */
			inc2 = inc2 + inc1;
			PSmoveto (0, inc2);
			counter++;
		} while (counter <=10);			/* max number of lines */
		
		PSmoveto(0,0);					/* moves to origin of rectangle */
		
		inc1 = (bounds.size.width/22);
		/* sets increment value to 1/22 of the width of rectangle */
	
		inc2 = 0;							/* initial increment value */
		counter = 0;						/* initialize counter */
		
		/* draws twenty two vertical lines across the rectangle */
		do {								    			PSlineto( inc2, bounds.size.height);
			/* draws vertical line */
			
			inc2 = inc2 + inc1;				/* increment by 1/22 the width of rectangle */
			PSmoveto (inc2, 0);			/* next vertical line */
			counter++;					/* increment counter */
		} while (counter <=24);
		PSstroke(); 						/* all drawn lines are visible */
		PSnewpath();						/* sets new drawing path */

}	/* -------------------end of drawing vertical and horizontal grids on screen------------------------- */
	

/*-----------------------  Signal Generator (waveform generation through two DACs) ----------------*/  
	
	/* executes only if any wave form is selected for generation */
	if ( waveSelected > 0)
{
	counter = 0;							/* initializes counter */
	angle =  (float) (circularSlider2Value);	/* takes starting angle of sine wave */
	dacCount =  (int)  (  freqSliderValue );	/* number of memory locations, controls frequency */
	
	DSPBootFile("/LocalApps/SignalLab.app/dacTalk.lod");
	/* Loads assembled object DSP file for signal generation */

	DSPSetHF0();				/* sets host flag for communication to assebmly program */
	DSPSetHF1();				/* sets host flag for communication to assebmly program */
	DSPWriteTX(dacSelected);		/* selects either of the two DACs to generate signals by */
								/* writing to the DSP  */
	DSPWriteTX(dacCount);		/* writes to DSP, the number of memory locations that */
								/* contain the look up table for signal generation */
	
	/*-------------------------------------  sine wave generation ------------------------------------------*/
	/* if wave selected is sine wave */
	
	/* Generates the look up table to generate a sine wave, mathematically computed, to the set 	* frequency and amplitude
	*/
	if (waveSelected == 1) {
	do
	{
		dataDSP[ counter ] =   (int ) ( (sin ( (angle * 3.141592654 ) /180 ) * 0x4444  )  * 		voltSliderValue);
		/* computes the sine, with the set frequency and amplitude and writes into buffer */	
		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */

		angle = (angle + (   (float ) ( circularSlider1Value) / ( float ) (dacCount) ) );
		/* increments angle, till the set ending angle */
	
		if (angle > 360.0)  { angle =  0.0;}
		/* calculates values till angle = 360 degrees and resets to zero */

		counter++;			/* increments counter */
	}while(counter <= dacCount);  }	/* repeats till set number of memory locations */
	
	/* -------------------------------------- end of sine wave generation ---------------------------------*/
	
	/* -------------------------------------- unipolar pulse wave generation ---------------------------------*/
	if (waveSelected == 2) {
	/* executes only if unipolar pulse wave is selected */
	
	SQvalue = 1;					/* a positive value to generate wave from 0 to +3 volts */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the 'high/low'  portion of the pulse wave to the set frequency, */
		/* amplitude and duty cycle*/
		
		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */

		if ( counter >= ( int ) ( dacCount * sqSliderValue ) ) {  SQvalue = 0;}
		/* checks if the 'low' portion of the pulse wave is reached */

		counter++;					/* increments counter */
	}while(counter <= dacCount ) ;  }	/* repeats till set number of memory locations */

	/* ---------------------------------- end of unipolar pulse wave generation -------------------------------*/	
	
	/* -------------------------------------- bipolar pulse wave generation ---------------------------------*/
	if (waveSelected == 3) {
	/* executes only if bipolar pulse wave is selected */

	SQvalue = 1;					/* a positive value to generate wave from 0 to +3 volts */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the 'high/low'  portion of the pulse wave to the set frequency, */
		/* amplitude and duty cycle*/
		
		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */
		
		if ( counter >  (dacCount * sqSliderValue) ) {  SQvalue = -1;}
		/* checks if the 'low' (negative) portion of the pulse wave is reached */
	
		counter++;				/* increments counter */
	}while(counter <= dacCount);  }	/* repeats till set number of memory locations */

	/* -----------------------------------end of bipolar pulse wave generation ------------------------------*/
	
	
	/* ------------------------------------------- DC positive generation --------------------------------------*/
	if (waveSelected == 4) {
	/* executes only if DC positive generation is selected */

	SQvalue = 1;				/* a positive value to generate wave from 0 to +3 volts */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the positive DC value, to the set amplitude */
	
		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */

		counter++;					/* increments counter */
	}while(counter <= dacCount +50) ;  }	/* repeats till set number of memory locations */
	
	/* -------------------------------------end of DC positive generation --------------------------------------*/

	/* ------------------------------------------- DC negative generation --------------------------------------*/
	if (waveSelected == 5) {
	/* executes only if DC negative generation is selected */

	SQvalue = -1;				/* a negative value to generate wave from 0 to -3 volts */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the negative DC value, to the set amplitude */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */

		counter++;					/* increments counter */
	}while(counter <= dacCount + 50);}	/* repeats till set number of memory locations */

	/* -------------------------------------end of DC negative generation -------------------------------------*/

	
	/* ----------------------------------------staircase wave generation ---------------------------------------*/
	if (waveSelected == 6) {
	/* executes only if staircase wave generation is selected */
	
	SQvalue = 0;						/* controls step height, proportional to any volts */
	acounter = 0;						/* initialize counter */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the step of the staircase */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes from the buffer to the DSP bootstrap program */

		acounter++;					/* increments counter */
		if ( acounter >  (int)    (dacCount / (staircaseStep * 10) )    )
		{  SQvalue = SQvalue +  (1 / ((staircaseStep * 10 ) - 1 ) ); acounter = 0;  }
		/* checks for the correct number of steps has reached, computes new value accordingly */
		
		counter++;					/* increments counter */
	}while(counter <= dacCount);  }		/* repeats till set number of memory locations */

	/* ---------------------------------- end of staircase wave generation ------------------------------------*/

	/* ------------------------------------unipolar ramp wave generation -------------------------------------*/
	if (waveSelected == 7) {
	/* executes only if unipolar ramp wave generation is selected */

	SQvalue = 0;				/* incremental value that holds the slope of the ramp */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the calculated value for the ramp wave, to the set frequency and */
		/* amplitude  */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes into buffer, the data for unipolar ramp generation */

		SQvalue = SQvalue + ( float ) ( 1.00 /  ( float )dacCount); 
		/* calculates incremental value for ramp wave */

		if ( counter >= ( dacCount  ) ) {  SQvalue = 0;}                           /* drop from peak */
		counter++;					/* increments counter */
	}while(counter <= dacCount ) ;  }	/* repeats till set number of memory locations */
	
	/* -------------------------------end of unipolar ramp wave generation --------------------------------*/

	/* ------------------------------------bipolar ramp wave generation -------------------------------------*/
	if (waveSelected == 8) {
	/* executes only if bipolar ramp wave generation is selected */

	SQvalue = -1;							/* initial negative value to start waveform */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the calculated value for the ramp wave, to the set frequency and */
		/* amplitude  */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes into buffer, the data for bipolar ramp generation */

		SQvalue = SQvalue + ( float ) ( 2.00 /  ( float )dacCount); 
		/* incremental value for bipolar ramp wave form */
	
		if ( counter >= ( dacCount  ) ) {  SQvalue = 0;}
		/* drop from peak to most negative portion of wave */

		counter++;						/* increments counter */
	}while(counter <= dacCount ) ;  }		/* repeats till set number of memory locations */

	/* ------------------------------ end of bipolar ramp wave generation -----------------------------------*/

	/* --------------------------------unipolar triangular  wave generation ----------------------------------*/
	if (waveSelected == 9) {
	/* executes only if unipolar triangular wave generation is selected */

	SQvalue = 0;						/* initial value for start of wave */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the calculated value for the unipolar triangular  wave, to the set */
 		/* frequency and amplitude  */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes into buffer, the data for unipolar triangular generation */

		if (counter < (int) ( (float) dacCount/2) ) {
		SQvalue = SQvalue + ( float ) ( 1.00 /  ( float ) (dacCount/2 ) );   }
		/* checks and increments if wave value has reached positive peak */

		if ( counter >=  (int) ( (float) dacCount/2  ) ) {  SQvalue = SQvalue - (float) (1.00/(float) 		(dacCount/2 ));}
		/* checks and decrements after wave has reached positive peak */		/* starts slope towards zero */		

		counter++;						/* increments counter */
	}while(counter <= dacCount ) ;  }		/* repeats till set number of memory locations */

	/* ---------------------------------bipolar triangular  wave generation ----------------------------------*/

	if (waveSelected == 10) {
	/* executes only if unipolar triangular wave generation is selected */

	SQvalue = -1;					/* initial negative start value for wave */
	 do
	{
		dataDSP[ counter ] =   (int )  (  (  SQvalue *  0x4444 )  * voltSliderValue);
		/* writes into buffer, the calculated value for the bipolar triangular  wave, to the set */
 		/* frequency and amplitude  */

		DSPWriteTX(dataDSP[ counter ]);
		/* writes into buffer, the data for bipolar triangular generation */

		if (counter < (int) ( (float) dacCount/2) ) 
		{ SQvalue = SQvalue + ( float ) ( 2.00 /  ( float ) (dacCount/2 ) );   }
		/* checks and increments if wave value has reached positive peak */

		if ( counter >=  (int) ( (float) dacCount/2  ) ) {  SQvalue = SQvalue - (float) (2.00/(float) 		(dacCount/2 ));}
		/* checks and decrements after wave has reached positive peak */		/* starts slope towards negative */	

		counter++;					/* increments counter */
	}while(counter <= dacCount ) ;  }	/* repeats till set number of memory locations */

	/* ------------------------------end of bipolar triangular  wave generation -----------------------------*/


	}				/* end of   ENTIRE  "if( wave Selected)"  */	
/*----------------End of  Signal Generator (waveform generation through two DACs) -----------------*/


/*------------------  capturing the waveform generated on the dual channel "scope" --------------------*/

		PSsetlinewidth( beamFocusValue);
		/* sets the trace beam width value */

		PSsethsbcolor ( traceHue,  traceSat, traceBrt);
		/* sets trace beam color to selected color */

		PSsetalpha( traceAlpha);
		/* sets trace beam transparency to selected value */

	/*-------------------------------------  Oscilloscope Channel 1 -------------------------------------------*/
	if(ch1Selected == 2)
	/* executes if input of channel 1 is connected to "ground" */
	
	{
			PSmoveto (0,  vpos1Value + vpos1FineValue );
			/* moves the trace beam to the set vertical position on the "scope" */

			PSlineto( ( bounds.size.width), vpos1Value + vpos1FineValue );
			/* draws a horizontal line to the edge of the "scope" to simulate a ground voltage */

			PSstroke(); 			/* drawn lines are visible */
			PSnewpath();			/* starts a new drawing path */
	}
	
	if (ch1Selected == 3)
	/* executes if input of channel 1 is connected to signal generator */
	
		{
		x2 = 0; y2 = 0; xpos = 0;		/* initial "scope" coordinate values */
		PSmoveto (0,  vpos1Value + vpos1FineValue );
		/* moves the trace beam to the set vertical position on the "scope" */

		counter = 0;					/* initializes counter */
		xpos =  counter;
		 	 do
			{
			x2 =  xpos + (float) ( (counter * (1.22222222/ freqScale1Factor )) ) ;
			/* sets the horizontal incremental value (frequency) to the set time */
			
			y2 =  (float) ( ( dataDSP[ counter]  * (0.000188827 *  vDiv1Value )) + ( vpos1Value + 			vpos1FineValue) );
			/* sets the vertical incremental value (amplitude) to the set amplitude */

			PSlineto (x2 , y2);
			PSmoveto (x2 , y2);
			/* draws the waveform on the scope in increments */

			if (counter >= (freqSliderValue )) { counter = 0; xpos = x2; }
			/* repeats drawing waveform after a complete cycle */

			counter = counter + freqScale1Factor;
			/* increments counter by a scaled value */

		}while( x2 <= bounds.size.width);	/* draws only in the view of the "scope" */
		
	}	
		
	if (ch1Selected == 4) 
	/* executes if input of channel 1 is connected to ADC  */
	{
		x2 = 0; y2 = 0; xpos = 0;		/* initial "scope" coordinate values */
		PSmoveto (0,  vpos1Value + vpos1FineValue );
		/* moves the trace beam to the set vertical position on the "scope" */

		counter = 0;					/* initializes counter */
		xpos =  counter;
		acounter = scrollValue ;		/* scrolling feature origin */
		
		y2 =  (float) ( ( dataDSPSample1[ acounter ]  * (0.000188827 *  vDiv1Value )) + ( 		vpos1Value + vpos1FineValue) );			/* trace beam displays the vertical position of the first sampled data, to the set amplitude */
		
		PSmoveto (x2 , y2);		/* moves to starting position of beam */
		do						/* loop to draw waveform on "scope" */
		{
		x2 =  (float) ( (counter * (27.4444444444/ freqScale1Factor )) ) ;
		/* calculates the horizontal (frequency) increment value, to the set time/Div */

		y2 =  (float) ( ( dataDSPSample1[ acounter]  * (0.000188827 *  vDiv1Value )) + ( 		vpos1Value + vpos1FineValue) );
		/* calculates the vertical (amplitude) increment value, to the set volt/Div */

			PSlineto (x2 , y2);
			PSmoveto (x2 , y2);
			/* draws the waveform on the scope in increments */

			counter = counter + 1;			/* increments counter */
			acounter++;					/* increments counter */
		}while( x2 <= bounds.size.width);	/* draws only in the view of the "scope" */
		PSstroke();						/* drawn lines are visible */
	}
	
	/*-------------------------------end of display on Oscilloscope Channel 1 -----------------------------*/	
	/*-------------------------------------  Oscilloscope Channel 2 -------------------------------------------*/		
		
	if(ch2Selected == 2)
	/* executes if input of channel 2 is connected to "ground" */
 
	{
			PSmoveto (0,  vpos2Value + vpos2FineValue );
			/* moves the trace beam to the set vertical position on the "scope" */

			PSlineto( ( bounds.size.width), vpos2Value + vpos2FineValue );
			/* draws a horizontal line to the edge of the "scope" to simulate a ground voltage */

			PSstroke();				/* drawn lines are visible */
			PSnewpath();				/* starts a new drawing path */
	}
	
	if (ch2Selected == 3) 
	/* executes if input of channel 2 is connected to signal generator */

	{
		x2 = 0; y2 = 0; xpos = 0;		/* initial "scope" coordinate values */
		PSmoveto (0,  vpos2Value + vpos2FineValue );
		/* moves the trace beam to the set vertical position on the "scope" */

		counter = 0;					/* initializes counter */
		xpos =  counter;
		 	 do
			{
			x2 =  xpos + (float) ( (counter * (1.22222222/ freqScale2Factor )) ) ;
			/* sets the horizontal incremental value (frequency) to the set time */
 
 			y2 =  (float) ( ( dataDSP[ counter]  * (0.000188827 *  vDiv2Value )) + ( vpos2Value + 			vpos2FineValue) );
			/* sets the vertical incremental value (amplitude) to the set amplitude */

			PSlineto (x2 , y2);
			PSmoveto (x2 , y2);
			/* draws the waveform on the scope in increments */

			if (counter >= (freqSliderValue )) { counter = 0; xpos = x2; }
			/* repeats drawing waveform after a complete cycle */

			counter = counter + freqScale2Factor;
			/* increments counter by a scaled value */

		}while( x2 <= bounds.size.width);	/* draws only in the view of the "scope" */
	
	}
	
	if (ch2Selected == 4)
	/* executes if input of channel 1 is connected to ADC  */
	{
		x2 = 0; y2 = 0; xpos = 0;			/* initial "scope" coordinate values */
		PSmoveto (0,  vpos2Value + vpos2FineValue );
		/* moves the trace beam to the set vertical position on the "scope" */

		counter = 0;						/* initializes counter */
		xpos =  counter;
		counter =  0 ;
		acounter = scrollValue ;			/* scrolling feature origin */
		
		y2 =  (float) ( ( dataDSPSample2[ acounter ]  * (0.000188827 *  vDiv2Value )) + ( 		vpos2Value + vpos2FineValue) );
		/* trace beam displays the vertical position of the first sampled data, to the set amplitude */
		
		PSmoveto (x2 , y2);				/* moves to starting position of beam */
		do								/* loop to draw waveform on "scope" */
		{
		x2 =  (float) ( (counter * (27.4444444444/ freqScale2Factor )) ) ;
		/* calculates the horizontal (frequency) increment value, to the set time/Div */
 
		y2 =  (float) ( ( dataDSPSample2[ acounter]  * (0.000188827 *  vDiv2Value )) + ( 		vpos2Value + vpos2FineValue) );
		/* calculates the vertical (amplitude) increment value, to the set volt/Div */

			PSlineto (x2 , y2);
			PSmoveto (x2 , y2);
			/* draws the waveform on the scope in increments */

			counter = counter + 1;			/* increments counter */
			acounter++;					/* increments counter */
		}while( x2 <= bounds.size.width);	/* draws only in the view of the "scope" */
		 PSstroke();						/* drawn lines are visible */
	}
	/*-------------------------------end of display on Oscilloscope Channel 2 --------------------------------*/
	
	/*------- displays the Time/Division and Volts/Division switch settings of the "scope " ----------*/

	PSstroke();
	PSnewpath();							/* new path for drawing */
	PSsethsbcolor ( textHue,  textSat, textBrt);
	/* sets text color to selected color */

	PSsetalpha( textAlpha);
	/* sets text transparency to selected alpha value */

	PSselectfont ("Times-Medium", 12.0);
	/* selects font type as Times-Roman with a 12 point size */

	PSmoveto( 25, 4);
	/* moves to lower left corner of the "scope " */

	PSshow ( voltString1 );
	/* displays Volt/Division value of channel 1 */

	PSmoveto( 25, 320);
	/* moves to lower right corner of the "scope " */

	PSshow ( voltString2 );
	/* displays Volt/Division value of channel 2 */
	
	PSmoveto( (bounds.size.width/2) + 25, 4);
	/* moves to upper left corner of the "scope " */

	PSshow (timeString1);
	/* displays Time/Division value of channel 1 */

	PSmoveto( (bounds.size.width/2) + 25, 320);
	/* moves to upper right corner of the "scope " */

	PSshow (timeString2);
	/* displays Time/Division value of channel 2 */

	return self; 						/* self is the return value of this method */ 
}

/*-----------------------------------------  End of drawself method ----------------------------------------------*/


- readADC:sender

/* Invoked when read (steady signal-volts) button is clicked.  It loads the assembled
* "adcListenOnce.lod" file into DSP ram and executes it.  The DSP program  samples the ADC
* AD7703 and updates the host register continuously.  The DSPReadRx reads this sampled data
* thrice, the first two reads are ignored to get rid of spurious data if any, and displays in a text field
* object.  The display value is in volts, after scaling the sampled data to read between 0 to 2.5 volts 
* Before it reads the data from the DSP , DSPSetHF0 and DSPSetHF1 enables host flags for
* communication to the DSP 
*/

{
    			int data1;						/* initializes variables */
			float data2;
			data2 = 0;
			DSPBootFile("/LocalApps/SignalLab.app/adcListenOnce.lod");
			/* Loads assembled object DSP file for steady signal capture */

			DSPSetHF0();		/* sets host flag for communication to assebmly program */
			DSPSetHF1();		/* sets host flag for communication to assebmly program */


			{
				DSPReadRX(&data1);	/* reads data from DSP, ignores */
				DSPReadRX(&data1);	/* reads data from DSP, ignores */
				DSPReadRX(&data1);	/* reads data from DSP */
				data2 = ( ( float ) data1 / 419430 );
				/* scales sampled data to 0 - 2.5 Volts range */

			}
		
				[adcOut setFloatValue:data2 at:0];
				/* displays the voltage value in a text field object */

	DSPClose();				/* close the DSP and clean up the device */
        [self display]; 			/* invokes drawself method */
	return self;				/* self is the return value of this method */ 
}

- wave1:sender

/* Invoked when sine button of the signal generator is clicked.  Sets a flag to indicate that sine wave 
* is selected for the signal generator.  Drawself method is invoked, which actually generates the sine
* signal to the set amplitude, frequency and starting/ending sine angle
*/
 
{
	waveSelected = 1;			/* selects sine wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave2:sender

/* Invoked when pulse (unipolar) button of the signal generator is clicked.  Sets a flag to indicate that 
* pulse (unipolar) wave is selected for the signal generator.  Drawself method is invoked, which 
* actually generates the pulse (unipolar) signal to the set amplitude, frequency and duty cycle
*/

{
	waveSelected = 2;			/* selects pulse (unipolar) wave, for the signal generator */
    	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave3:sender

/* Invoked when pulse (bipolar) button of the signal generator is clicked.  Sets a flag to indicate that 
* pulse (bipolar) wave is selected for the signal generator.  Drawself method is invoked, which 
* actually generates the pulse (bipolar) signal to the set amplitude, frequency and duty cycle
*/

{
	waveSelected = 3;			/* selects pulse (bipolar) wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave4:sender

/* Invoked when DC (positive) button of the signal generator is clicked.  Sets a flag to indicate that 
* DC (positive)  is selected for the signal generator.  Drawself method is invoked, which  actually 
* generates the DC positive signal to the set amplitude
*/

{
	waveSelected = 4;			/* selects DC positive, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave5:sender

/* Invoked when DC (negative) button of the signal generator is clicked.  Sets a flag to indicate that 
* DC (negative)  is selected for the signal generator.  Drawself method is invoked, which  actually 
* generates the DC negative signal to the set amplitude
*/

{
	waveSelected = 5;			/* selects DC negative, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave6:sender

/* Invoked when staircase button of the signal generator is clicked.  Sets a flag to indicate that 
* staircase wave is selected for the signal generator.  Drawself method is invoked, which actually 
* generates the staircase signal to the set amplitude, frequency and number of steps
*/

{
	waveSelected = 6;			/* selects staircase wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave7:sender

/* Invoked when ramp (unipolar) button of the signal generator is clicked.  Sets a flag to indicate that 
* ramp (unipolar) wave is selected for the signal generator.  Drawself method is invoked, which 
* actually generates the ramp (unipolar) signal to the set amplitude and frequency
*/

{
	waveSelected = 7;			/* selects ramp (unipolar) wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave8:sender

/* Invoked when ramp (bipolar) button of the signal generator is clicked.  Sets a flag to indicate that 
* ramp (bipolar) wave is selected for the signal generator.  Drawself method is invoked, which 
* actually generates the ramp (bipolar) signal to the set amplitude and frequency
*/

{
 	waveSelected = 8;			/* selects ramp (bipolar) wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}

- wave9:sender

/* Invoked when triangular (unipolar) button of the signal generator is clicked.  Sets a flag to indicate
* that triangular (unipolar) wave is selected for the signal generator.  Drawself method is invoked,
* which actually generates the triangular (unipolar) signal to the set amplitude and frequency
*/

{
	waveSelected = 9;			/* selects triangular (unipolar) wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}


- wave10:sender

/* Invoked when triangular (bipolar) button of the signal generator is clicked.  Sets a flag to indicate
* that triangular (bipolar) wave is selected for the signal generator.  Drawself method is invoked,
* which actually generates the triangular (bipolar) signal to the set amplitude and frequency
*/

{
	waveSelected = 10;		/* selects triangular (bipolar) wave, for the signal generator */
	[self display];			/* invokes drawself method */;
	return self;				/* self is the return value of this method */
}


- selectDac1:sender

/* Invoked when "DAC 1" button is clicked.  Selects one of the two DAC to generate or reproduce
* generated or captured signals.  Drawself method is executed which actually takes the selected
* DAC and passes the information to the DSP program
*/

{
	dacSelected = 0x00E;		/* hex value which selects DAC #1 */
        [self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}


- selectDac2:sender

/* Invoked when "DAC 2" button is clicked.  Selects one of the two DAC to generate or reproduce
* generated or captured signals.  Drawself method is executed which actually takes the selected
* DAC and passes the information to the DSP program
*/

{
	dacSelected = 0x016;		/* hex value which selects DAC #2 */
        [self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}


- freqSlider:sender

/* Invoked when the frequency slider of the signal generator is moved.  Frequency is controlled by
* varying the number of memory locations that contains the look up data for signal generation.  
* Higher the number of memory locations, lower will be the generated frequency.  The lowest number
* of memory locations is 135 (100 Hz) while the highest is 13500 (1 Hz).  Hence the variable
* 'freqSliderValue' contains the number of memory locations that controls frequency.  The DSP 
* needs extra (32K) RAM to store look up data for the signal generator.  The variable
* 'freq1To17Value' indicates one of the 9 frequency to be generated.  The variable 
* 'freqDisplayValue'    displays on a text field, the actual frequency generated.  This method only
* sets the variables to the selected frequency, after which drawself method is executed.
*/

{
	float  freqDisplayValue;	/* variable used to store the actual frequency being generated */
	freqDisplayValue = 0;		/* Initialize */
	
	freq1To17Value = [sender intValue];
	/* reads slider value from slider object, the 'sender' of the message which invokes this */
	/*  method */
	
	if (freq1To17Value == 1 )  {  freqSliderValue = 13500;   freqDisplayValue = 1;     }
	/* selection of 1 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 2 )  {  freqSliderValue = 5400;       freqDisplayValue = 2.5;     }
	/* selection of 2.5 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 3 )  {  freqSliderValue = 2700;       freqDisplayValue = 5;     }
	/* selection of 5 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 4 )  {  freqSliderValue = 1800;       freqDisplayValue = 7.5;     }
	/* selection of 7.5 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 5 )  {  freqSliderValue = 1350;       freqDisplayValue = 10;  }
	/* selection of 10 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 6 )  {  freqSliderValue = 540;         freqDisplayValue = 25;   }
	/* selection of 25 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 7 )  {  freqSliderValue = 270;         freqDisplayValue = 50;   }
	/* selection of 50 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 8 )  {  freqSliderValue = 180;         freqDisplayValue = 75;   }
	/* selection of 75 Hz frequency to be generated by the signal generator */

	if (freq1To17Value == 9 )  {  freqSliderValue = 135;         freqDisplayValue = 100;    }
	/* selection of 100 Hz frequency to be generated by the signal generator */

	[freqOut setFloatValue:freqDisplayValue at:0];
	/* displays the selected frequency on the text field object */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}


- voltSlider:sender

/* Invoked when the amplitude slider of the signal generator is moved.  The slider has a total of 
* 300 steps, corresponding to 300 increments of amplitude from -3 to 3 volts. Each step increments
* or decrements the amplitude by 0.02 volts.  When the slider is moved, the correct scaled value to
* be displayed is written to a text field object.  Drawself method is executed which actually makes
* the corresponding change to the amplitude of the wave bring generated 
*/

{
	int voltOldValue;			/* variable */
	float voltNewValue;		/* variable */
	
	voltOldValue = [sender intValue];
	/* reads slider value from slider object, the 'sender' of the message which invokes this */
	/* method */

	voltSliderValue = ( (float)voltOldValue  / 10 );
	/* scales down the slider value by a factor  of 10, this is the actual value that will be used */
	/* by the drawself method to change the amplitude of signal being generated */

	voltNewValue = ( voltSliderValue / 10);
	/* further scaling down by a factor of 10, to be displayed as actual volts */

	[voltOut setFloatValue:voltNewValue at:0];
	/* displays the change in slider value in volts, on a text field object */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}


- swControl1:sender

/* Invoked when the End circular slider of Sine Wave control is moved.  Sets the ending angle
* of the sine wave to be generated.  The starting and ending degrees value of sine wave signal 
* generator can be controlled from 0 - 360 degrees.  The method displays the angle (degrees) in a
*  text field object.  Drawself method is executed which takes the ending value of the sine wave
* and then generates to the set amplitude, frequency and starting angle
*/

{
		circularSlider1Value = [sender intValue];
		/* reads slider value from circular slider object, the 'sender' of the message which  */
		/* invokes this  method */

		[swControl1Out setIntValue:circularSlider1Value at:0];
		/* displays the ending angle in degrees in a text field object */

        	[self display];		/* invokes drawself method */; 
		return self;			/* self is the return value of this method */
}

- swControl2:sender

/* Invoked when the Begin circular slider of Sine Wave control is moved.  Sets the starting angle
* of the sine wave to be generated.  The starting and ending degrees value of sine wave signal 
* generator can be controlled from 0 - 360 degrees.  The method displays the angle (degrees) in a
*  text field object.  Drawself method is executed which takes the starting value of the sine wave
* and then generates to the set amplitude, frequency and ending angle
*/

{
		circularSlider2Value = [sender intValue];
		/* reads slider value from circular slider object, the 'sender' of the message which  */
		/* invokes this  method */

		[swControl2Out setIntValue:circularSlider2Value at:0];
		/* displays the starting angle in degrees in a text field object */

       	 	[self display];		/* invokes drawself method */; 
		return self;			/* self is the return value of this method */
}

- sqControl:sender

/* Invoked when the Pulse Control slider of the signal generator is moved.  Sets the duty cycle
* between 0 to 100 % in incremental steps of 1 %.  The sets duty cycle in percent is displayed 
* in a text field object.  Drawself method is executed which takes the duty cycle value and
* generates the pulse wave with the set amplitude and frequency
*/

{
		int temp;				/* initialization, temporary variable */
		sqSliderValue = [sender floatValue];
		/* reads slider value from slider object, the 'sender' of the message which invokes this */
		/* method */

		temp = (int) (sqSliderValue * 100);
		/* scales read slider value to be in the range of 0 to 100 % */

		[sqControlOut setIntValue:temp at:0];
		/* displays the set duty cycle value in percent in a text field object */

        	[self display];		/* invokes drawself method */; 
		return self;			/* self is the return value of this method */
}

- staircaseControl:sender

/* Invoked when the Staircase Control slider of the signal generator is moved.  Sets the number
* of steps in a staircase wave to be generated.  The range of steps is between 0 to 10.  The
* number of steps set is displayed in a text field object.  Drawself method is executed which
* takes the value of number of steps and generates a staircase signal to the set frequency and
* amplitude 
*/

{
		int temp;				/* initialization, temporary variable */
		temp = [sender intValue];
		/* reads slider value from slider object, the 'sender' of the message which invokes this */
		/* method */

		staircaseStep = (float) ( (float) temp/10 );
		/* scales down the slider value by a factor of 10 */

		[staircaseControlOut setIntValue:temp at:0];
		/* displays the set number of steps in a text field object */

        	[self display];		/* invokes drawself method */; 
		return self;			/* self is the return value of this method */
}

-gridSwitch:sender

/* Invoked when the Grid button is clicked.  Sets a variable gridFlag, that enables or disables
* the grid to be displayed on the "scope". Drawself method is executed which actually draws
* the grid or does not, depending on the gridFlag variable
*/

{
	gridFlag = !gridFlag;				/* complements the previous contents of itself */
	[self display];					/* invokes drawself method */; 
	return self;						/* self is the return value of this method */
}


- backColor:sender

/* Invoked when a color 'chip' is dropped into the background color well, from the color wheel.
* Receives the raw value from the color well and converts it into Hue, Saturation , Brightness and
* Alpha values, stores them in corresponding variables.  Drawself method is executed which 
* updates the background color of the "scope"
*/

{
	NXColor color;					/* color variable is NXColor type */
	
	color =[sender color];
	/* reads value from background color well object, the 'sender' of the message which invokes */
	/* this method */

	NXConvertColorToHSBA(color, &backHue, &backSat, &backBrt, &backAlpha);
	/* converts value read from color well into HSBA format */

	[self display];				/* invokes drawself method */; 
	return self;					/* self is the return value of this method */
}

- gridColor:sender

/* Invoked when a color 'chip' is dropped into the grid color well, from the color wheel.  Receives
* the raw value from the color well and converts it into Hue, Saturation , Brightness and
* Alpha values, stores them in corresponding variables.  Drawself method is executed which 
* updates the grid color of the "scope"
*/

{
	NXColor color;				/* color variable is NXColor type */

	color =[sender color];
	/* reads value from grid color well object, the 'sender' of the message which invokes */
	/* this method */

	NXConvertColorToHSBA(color, &gridHue, &gridSat, &gridBrt, &gridAlpha);
	/* converts value read from color well into HSBA format */

	[self display];				/* invokes drawself method */; 
	return self;					/* self is the return value of this method */
}

- traceColor:sender

/* Invoked when a color 'chip' is dropped into the trace color well, from the color wheel.  Receives
* the raw value from the color well and converts it into Hue, Saturation , Brightness and
* Alpha values, stores them in corresponding variables.  Drawself method is executed which 
* updates the trace beam color of the "scope"
*/

{
	NXColor color;				/* color variable is NXColor type */
	color =[sender color];
	/* reads value from trace color well object, the 'sender' of the message which invokes */
	/* this method */

	NXConvertColorToHSBA(color, &traceHue, &traceSat, &traceBrt, &traceAlpha);
	/* converts value read from color well into HSBA format */

	[self display];				/* invokes drawself method */; 
	return self;					/* self is the return value of this method */
}

- textColor:sender

/* Invoked when a color 'chip' is dropped into the text color well, from the color wheel.  Receives
* the raw value from the color well and converts it into Hue, Saturation , Brightness and
* Alpha values, stores them in corresponding variables.  Drawself method is executed which 
* updates the text color of the "scope"
*/

{
	NXColor color;				/* color variable is NXColor type */
	color =[sender color];
	/* reads value from text color well object, the 'sender' of the message which invokes */
	/* this method */

	NXConvertColorToHSBA(color, &textHue, &textSat, &textBrt, &textAlpha);
	/* converts value read from color well into HSBA format */

	[self display];				/* invokes drawself method */; 
	return self;					/* self is the return value of this method */
}

- calibrate:sender

/* Invoked when the Calibrate button is clicked.  Calibrates the ADC AD7703 by loading the DSP
* bootstrap program 'adcCalibrate.lod'.  The DSP program then calibrates the ADC by activating 
* the calibrate pin of the ADC.  Executes drawself method
*/

{
	DSPBootFile("/LocalApps/SignalLab.app/adcCalibrate.lod");
	/* Loads assembled object DSP file for calibrating the ADC */

	DSPClose();
	/* close the DSP and clean up the device */

        [self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv11:sender

/* Invoked when '1 V' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 1 Volt/Division on the amplitude axis
*/

{
        vDiv1Value = 1.00;		/* sets the scaling value for the selection */
	
	voltString1 = "CH1: 1 Volts / Division";
	/* title string to denote amplitude selection */
	
	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv12:sender

/* Invoked when '0.1 V' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.1 Volt/Division on the amplitude axis
*/

{
        vDiv1Value = 10.00;		/* sets the scaling value for the selection */
	
	voltString1 = "CH1: 100 mVolts / Division";
	/* title string to denote amplitude selection */

        [self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv13:sender

/* Invoked when '0.01 V' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.01 Volt/Division on the amplitude axis
*/

{
         vDiv1Value = 100.00;		/* sets the scaling value for the selection */
	 
	voltString1 = "CH1: 10 mVolts / Division";
	/* title string to denote amplitude selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv21:sender

/* Invoked when '1 V' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 1 Volt/Division on the amplitude axis
*/

{
        vDiv2Value = 1.00;		/* sets the scaling value for the selection */
	
	voltString2 = "CH2: 1 Volts / Division";
	/* title string to denote amplitude selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv22:sender

/* Invoked when '0.1 V' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.1 Volt/Division on the amplitude axis
*/

{
        vDiv2Value = 10.00;		/* sets the scaling value for the selection */
	
	voltString2 = "CH2: 100 mVolts / Division";
	/* title string to denote amplitude selection */

        [self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vDiv23:sender

/* Invoked when '0.01 V' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* amplitude scaling value to be displayed on the "scope".  A title to denote the volt/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.01 Volt/Division on the amplitude axis
*/

{
         vDiv2Value = 100.00;		/* sets the scaling value for the selection */
	
	voltString2 = "CH2: 10 mVolts / Division";
	/* title string to denote amplitude selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv11:sender

/* Invoked when '2 mS' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 2 mSec/Division on the time axis
*/

{
	 freqScale1Factor = 1;		/* sets the scaling value for the selection */

	timeString1 = "CH1: 2 mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv12:sender

/* Invoked when '5 mS' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 5 mSec/Division on the time axis
*/

{
         freqScale1Factor = 2.5;	/* sets the scaling value for the selection */

	timeString1 = "CH1: 5 mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv13:sender

/* Invoked when '10 mS' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 10 mSec/Division on the time axis
*/

{
         freqScale1Factor = 5;		/* sets the scaling value for the selection */

	timeString1 = "CH1: 10 mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv14:sender

/* Invoked when '20 mS' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 20 mSec/Division on the time axis
*/

{
         freqScale1Factor = 10;		/* sets the scaling value for the selection */

	timeString1 = "CH1: 20 mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv15:sender

/* Invoked when '50 mS' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 50 mSec/Division on the time axis
*/

{
         freqScale1Factor = 25;		/* sets the scaling value for the selection */

	timeString1 = "CH1: 50 mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv16:sender

/* Invoked when '0.1 S' button of the Channel 1 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.1 Sec/Division on the time axis
*/

{
         freqScale1Factor = 50;		/* sets the scaling value for the selection */

	timeString1 = "CH1: 0.1 Sec / Division";
	/* title string to denote time selection */

 	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv21:sender

/* Invoked when '2 mS' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 2 mSec/Division on the time axis
*/

{
         freqScale2Factor = 1;		/* sets the scaling value for the selection */

	timeString2 = "CH2: 2  mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv22:sender

/* Invoked when '5 mS' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 5 mSec/Division on the time axis
*/

{
         freqScale2Factor = 2.5;	/* sets the scaling value for the selection */

	timeString2 = "CH2: 5  mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv23:sender

/* Invoked when '10 mS' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 10 mSec/Division on the time axis
*/

{
         freqScale2Factor = 5;		/* sets the scaling value for the selection */

	timeString2 = "CH2: 10  mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv24:sender

/* Invoked when '20 mS' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 20 mSec/Division on the time axis
*/

{
         freqScale2Factor = 10;		/* sets the scaling value for the selection */

	timeString2 = "CH2: 20  mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv25:sender

/* Invoked when '50 mS' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 50 mSec/Division on the time axis
*/

{
         freqScale2Factor = 25;		/* sets the scaling value for the selection */

	timeString2 = "CH2: 50  mSec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- tDiv26:sender

/* Invoked when '0.1 S' button of the Channel 2 "scope" is clicked.  Sets a variable to appropriate
* time scaling value to be displayed on the "scope".  A title to denote the time/div selected is
* also set.  Drawself method is then executed which re-draws the waveform being either generated or 
* reproduced to show 0.1 Sec/Division on the time axis
*/

{
         freqScale2Factor = 50;		/* sets the scaling value for the selection */

	timeString2 = "CH2: 0.1  Sec / Division";
	/* title string to denote time selection */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vpos1:sender

/* Invoked when 'Coarse' slider of Channel 1 "scope" is moved.  This method sets the starting
* vertical position of the wave being generated or reproduced on the "scope".  Drawself method
* is executed which actually adjusts the vertical position of the trace beam of Channel 1
*/

{
	vpos1Value = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vpos2:sender

/* Invoked when 'Coarse' slider of Channel 2 "scope" is moved.  This method sets the starting
* vertical position of the wave being generated or reproduced on the "scope".  Drawself method
* is executed which actually adjusts the vertical position of the trace beam of Channel 2
*/

{
	vpos2Value = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vpos1Fine:sender

/* Invoked when 'Fine' slider of Channel 1 "scope" is moved.  This method adjusts the fine
* vertical position of the wave being generated or reproduced on the "scope".  Drawself method
* is executed which actually adjusts the fine vertical position of the trace beam of Channel 1
*/

{
	vpos1FineValue = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- vpos2Fine:sender

/* Invoked when 'Fine' slider of Channel 2 "scope" is moved.  This method adjusts the fine
* vertical position of the wave being generated or reproduced on the "scope".  Drawself method
* is executed which actually adjusts the fine vertical position of the trace beam of Channel 2
*/

{
	vpos2FineValue = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method */

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- ch1Input1:sender

/* Invoked when the Input 'OFF' button of Channel 1 "scope" is clicked.  Sets a variable to 
* indicate no input connection to channel 1.  The "scope" does not display any waveform.
* Drawself method is executed which uses the variable set in this method, not to display
* any waveform on the "scope"
*/

{
	ch1Selected = 1;			/* sets variable to indicate no input connection */
	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- ch1Input2:sender

/* Invoked when the Input 'GND' button of Channel 1 "scope" is clicked.  Sets a variable to 
* indicate no ground connection to channel 1.  The "scope" displays a simulated 'ground' line.
* Drawself method is executed which uses the variable set in this method, to simulate a
* 'ground' line on the "scope"
*/

{
	ch1Selected = 2;		/* sets variable to indicate no ground connection */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch1Input3:sender

/* Invoked when the Input 'DAC' button of Channel 1 "scope" is clicked.  Sets a variable to 
* indicate connection to DAC on channel 1.  The "scope" displays waveforms being generated or 
* reproduced on this channel.  Drawself method is executed which uses the variable set in this
* method, to simulate the DAC connection
*/

{
	ch1Selected = 3;		/* sets variable to indicate connection to DAC */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch1Input4:sender

/* Invoked when the Input 'ADC' button of Channel 1 "scope" is clicked.  Sets a variable to 
* indicate connection to ADC on channel 1.  Only the contents of 'Sample1', a buffer used to store 
* captured data is displayed on channel 1.  The "scope" displays waveforms being captured or 
* reproduced on this channel.  Drawself method is executed which uses the variable set in this
* method, to simulate the ADC connection
*/

{
	ch1Selected = 4;		/* sets variable to indicate connection to ADC */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch2Input1:sender

/* Invoked when the Input 'OFF' button of Channel 2 "scope" is clicked.  Sets a variable to 
* indicate no input connection to channel 2.  The "scope" does not display any waveform.
* Drawself method is executed which uses the variable set in this method, not to display
* any waveform on the "scope"
*/

{
	ch2Selected = 1;		/* sets variable to indicate no input connection */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch2Input2:sender

/* Invoked when the Input 'GND' button of Channel 2 "scope" is clicked.  Sets a variable to 
* indicate no ground connection to channel 2.  The "scope" displays a simulated 'ground' line.
* Drawself method is executed which uses the variable set in this method, to simulate a
* 'ground' line on the "scope"
*/

{
	ch2Selected = 2;		/* sets variable to indicate no ground connection */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch2Input3:sender

/* Invoked when the Input 'DAC' button of Channel 2 "scope" is clicked.  Sets a variable to 
* indicate connection to DAC on channel 2.  The "scope" displays waveforms being generated or 
* reproduced on this channel.  Drawself method is executed which uses the variable set in this
* method, to simulate the DAC connection
*/

{
	ch2Selected = 3;		/* sets variable to indicate connection to DAC */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- ch2Input4:sender

/* Invoked when the Input 'ADC' button of Channel 2 "scope" is clicked.  Sets a variable to 
* indicate connection to ADC on channel 2.  Only the contents of 'Sample2', a buffer used to store 
* captured data is displayed on channel 2.  The "scope" displays waveforms being captured or 
* reproduced on this channel.  Drawself method is executed which uses the variable set in this
* method, to simulate the ADC connection
*/

{
	ch2Selected = 4;		/* sets variable to indicate connection to ADC */
	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- beamFocus:sender

/* Invoked when Focus slider is moved. Sets a variable that increases or decreases the trace
* beam of both channels.  Simulates the function of a 'Focus' knob found on oscilloscopes.  
* Drawself method is executed which uses this variable to adjust the beam thickness
*/
 
{
	beamFocusValue = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method.  The value returned from the slider object is in the range of 1 to 6, which is */
	/* directly used with PSSetlinewidth() function to set the line thickness*/

	[self display];			/* invokes drawself method */; 
	return self;				/* self is the return value of this method */
}

- bright:sender

/* Invoked when Brightness slider is moved. Sets a variable that increases or decreases the trace
* beam brightness of both channels.  Simulates the function of a 'Intensity' knob found on 
* oscilloscopes. Drawself method is executed which uses this variable to adjust the beam brightness
*/

{
	traceBrt = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method.  The value returned from the slider object is in the range of 0.5 to 1, which is */
	/* directly used with NXConvertColorToHSBA() function (only the brightness is varied) */

	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- gridLight:sender

/* Invoked when Grid Light slider is moved. Sets a variable that increases or decreases the grid
* brightness of the "scope".  Simulates the function of a light shining through the sides of a
* oscilloscope. Drawself method is executed which uses this variable to adjust the grid brightness
*/

{
	gridBrt = [sender floatValue];
	/* reads value from the slider object, the 'sender' of the message which invokes */
	/* this method.  The value returned from the slider object is in the range of 0 to 1, which is */
	/* directly used with NXConvertColorToHSBA() function (only the brightness is varied) */

	[self display];		/* invokes drawself method */; 
	return self;			/* self is the return value of this method */
}

- resetCursorRects

/* This is a overriden view method, automatically invoked to reset cursor rectangle */

{
	[self addCursorRect:&bounds cursor:crossHairCursor];
	/* message to view object to set cursor rectangle, so that the crosshair shows up in */
	/* signalView */

	return self;			/* self is the return value of this method */
}

- mouseDown: (NXEvent *) theEvent

/* Invoked every time the mouse button is pressed down inside signalView.  Responds to a 
* message whenever the user presses the mouse button when the cursor is over signalView.  The 
* cursor is changed to a cross-hairs image and starts receiving mouse dragged events.  Two sets of
* points are registered when the mouse cursor is on signalView - the first time when the uses clicks
* the mouse button, and then drags it and releases the mouse button.  The click and release points
* are registered to compose a rectangle and used in the trace feature to find amplitude and period 
* of the enclosed rectangle.  The rectangle would enclose a portion of a waveform whose amplitude 
* and period need to be found out.  Using the two sets of registered points, and using the time/div,
* volt/division variables, the amplitude and period of the rectangle is calculated and written to a text 
* field object to display the result in volts and time (seconds)
*/

{
	NXPoint currentPos, oldPos;	/* type NXPoint, each variable has both a x and y component */
	NXEvent *nextEvent;
	BOOL looping = 1;					/* True flag */
	int	oldMask;						/* mouse event masks */
	int	checkMask;					/* mouse event masks */
	float tempVar1, tempVar2;			/* temporary variables */
	tempVar1 =0.0;					/* initialization */
	tempVar2 = 0.0;					/* initialization */
	oldMask = [window eventMask];	/* stores current window events in oldMask */
	checkMask = NX_MOUSEUPMASK | NX_MOUSEDRAGGEDMASK;
	[window setEventMask: (oldMask | checkMask)];
	
	[self lockFocus];					
	/* used for dynamic drawing, in response to user action. Locks the PS focus on signalView  */
	/* prior to any drawing */
	
	oldPos = theEvent -> location;		/* registers first set of points  */
	
	[self convertPoint: &oldPos fromView:nil];
	/* conversion of oldPos from window coordinates to signalView coordinates */
	
	PSsetlinewidth (1.0);				/* linewidth for drawing markers on scope */
	PSsetalpha( 0.35);					/* transparency, can see through the drawing */
	PSsetgray( NX_LTGRAY);			/* color for drawing */
	
	/* while mouse button is held down */ 
	while (looping) {
	 	
			nextEvent = [NXApp getNextEvent: checkMask];
			looping = (nextEvent -> type != NX_MOUSEUP);
			/* if mouseup, looping is false */

			currentPos = nextEvent -> location;
			[self convertPoint: &currentPos fromView:nil];
			/* registers second set of coordinates and converts them from window to signalView */
			/* coordinates */
								
			/* if still in the same position */
			if ((currentPos.x == oldPos.x) | (currentPos.y == oldPos.y))
			{
									
			/* draws a cross-hair to mark the first set of coordinates on the scope, extending to */
			/* the ends of the signalView */
			
			PSnewpath();					/* new path for drawing */
			PSmoveto (oldPos.x, oldPos.y);	/* moves to the first registered point */
			PSlineto ( oldPos.x, bounds.size.height);  /* line from center to top of view */
			PSmoveto (oldPos.x, oldPos.y);	/* moves to the first registered point */
			PSlineto ( oldPos.x, 0.0);		/* line from center to bottom of view */
			PSmoveto (oldPos.x, oldPos.y);	/* moves to the first registered point */
			PSlineto ( 0.0, oldPos.y);		/* line from center to left of view */
			PSmoveto (oldPos.x, oldPos.y);	/* moves to the first registered point */
			PSlineto ( bounds.size.width, oldPos.y); /* line from center to right */
			PSstroke();					/* displays the cross-hairs */
						
			[window flushWindow];		/* refresh drawing  */
			}
							
		}
		
		/* draws a cross-hair to mark the second set of coordinates on the scope, extending to */
		/* the ends of the signalView */
			PSnewpath();					/* new path for drawing */
			PSmoveto (currentPos.x, currentPos.y);  /* moves to the second registered point */
			PSlineto ( currentPos.x, bounds.size.height);   /* line from center to top of view */
			PSmoveto (currentPos.x, currentPos.y);  /* moves to the second registered point */
			PSlineto ( currentPos.x, 0.0);	/* line from center to bottom of view */
			PSmoveto (currentPos.x, currentPos.y);  /* moves to the second registered point */
			PSlineto ( 0.0, currentPos.y);	/* line from center to left of view */
			PSmoveto (currentPos.x, currentPos.y);  /* moves to the second registered point */
			PSlineto ( bounds.size.width, currentPos.y);  /* line from center to right */
			PSstroke();					/* displays the cross-hairs */
			[window flushWindow];		/* refresh drawing  */			
			PScompositerect( oldPos.x, oldPos.y,  ( currentPos.x - oldPos.x), ( currentPos.y - 			oldPos.y) , NX_DOUT);
			/* draw a composite rectangle, with transparency, so that destination image */ 
			/* can be seen through source image.  This is a transparent rectangle with the two  */
			/* set of registered points as diagonals */
			
			PSclosepath(); 			/* closes drawing path */
			PSstroke();				/* displays drawing */
			PSfill();					/* fills the rectangle */
			tempVar1 = (float) ( abs ( oldPos.x - currentPos.x) ) / (33.0);
			/* period calculation of both channels, from the trace rectangle */

			tempVar2 =  (float ) (abs (oldPos.y - currentPos.y) )  /  (33.0) ;
			/* amplitude calculation of both channels, from the trace rectangle */
					
			[traceMatrix setFloatValue:0 at:0];			/* display zero at trace amplitude field, channel 1 */

			[traceMatrix setFloatValue:0 at:1];
			/* display zero at trace period field, channel 1 */

			[traceMatrix setFloatValue:0 at:2];
			/* display zero at trace amplitude field, channel 2 */

			[traceMatrix setFloatValue:0 at:3];
			/* display zero at trace period field, channel 2 */
					
		if( ch1Selected > 1) 
		{
		[traceMatrix setFloatValue:(tempVar2 / vDiv1Value) at:0];
		/* if channel 1 is ON, display amplitude in volts */

 		[traceMatrix setFloatValue:( (tempVar1 * 0.00002 * freqScale1Factor) * 100) at:1];
		/* if channel 1 is ON, display period in seconds */
		}
									
		if( ch2Selected > 1) 
		{
		[traceMatrix setFloatValue:(tempVar2 / vDiv2Value) at:2]; 
		/* if channel 2 is ON, display amplitude in volts */
    
		[traceMatrix setFloatValue:( (tempVar1 * 0.00002 * freqScale2Factor) *100 ) at:3];
		/* if channel 2 is ON, display period in seconds */
		}
									
		[window flushWindow];			/* refresh drawing */
		[self unlockFocus];				/* unlocks, lockfocus */
		[window setEventMask: oldMask];	/* restores oldMask */
		return self;						/* self is the return value of this method */
}

- clearTraceRect:sender

/* Invoked when the Clear button of the trace feature is clicked.  Since the trace feature uses
* dynamic drawing (without using drawself method), by executing drawself method, the 
* drawing by mouse is completely erased 
*/

{
	[self display];						/* invokes drawself method */;
	return self;							/* self is the return value of this method */
}

- defaultColor:sender

/* Invoked when the Default Colors button is clicked.  Restores the original colors of background,
* grid, trace beam and text on the "scope".  Changes the variables that controls the color components.
* Drawself method is executed which actually changes the color, using the variables.
*/

{
	PSsetalpha(1.0);					/* resets transparency value */
	backHue = 0.5385; backSat = 0.9333;  backBrt = 0.3250; backAlpha = 1;
	/* background color components to set color to dark blue */

	gridHue = 0.1803; gridSat = 0.5725;  gridBrt = 0; gridAlpha = 1;
	/* grid color components to set color to black */

	traceHue = 0.3836; traceSat = 0.9490;  traceBrt = 1; traceAlpha = 1;
	/* trace beam color components to set color to light blue */

	textHue = 0.5409; textSat = 0.733;  textBrt = 0.6666; textAlpha = 1;
	/* text color components to set color to dark blue, a shade darker than background */

	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- sample:sender

/* Invoked when the Sample button is clicked.  Loads the DSP bootstrap program which reads
* the ADC and stores in the DSP RAM.  The number of samples is selected through a slider
* object and displayed in a text field object.  The data displayed on the textfield object is collected
* and used to sample the ADC.  The sampled data is retrieved from the DSP RAM and stored either 
* in Sample1 buffer or Sample2 buffer, whichever is selected.  The minimum number of samples is 25
* while the maximum is 20,000.
*/

{
	int  counter;
	counter = 0;						/* counter initialization */
	maxSamplePoints = [sOut intValueAt:0];
	printf( "Max Sample Ponts = %i \n", maxSamplePoints );
	/* the number of data to be sampled is collected from the text field object */

			DSPBootFile("/LocalApps/SignalLab.app/adcListen.lod");
			/* Loads the DSP bootstrap program which samples the ADC */

			DSPSetHF0();			/* sets host flag for communication */
			DSPSetHF1();			/* sets host flag for communication */
			
	maxSamplePoints = maxSamplePoints + 100;
	/* samples extra 100 points */

	DSPWriteTX( maxSamplePoints ); 
	/* writes the amount of samples to DSP program */

	/* checks and executes loop if data is to be stored in Sample1 buffer */
	if ( storeSampleValue == 1)
	{
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			DSPReadRX(&dataDSPSample1[counter]);
			/* reads 7 samples and discards, spurious data */
	
		/* Loop to transfer samples data from DSP RAM to Sample1 buffer */
		do
		{
			DSPReadRX(&dataDSPSample1[counter]);
			/* reads sampled data from DSP RAM and writes into Sample1 buffer */

			dataDSPSample1[ counter] =  (int ) (dataDSPSample1[ counter ] /2.40);
			/* scales data to range 0 to 2.5 volts */

			counter++;			/* increments counter */
		} while (counter <= maxSamplePoints);  /* repeats loop till all data is transfered */
		
		/* loop to fill the remaining empty or previously used buffer with zeroes */
		do
		{
			dataDSPSample1[counter] = 0;
			/* writes zeroes to buffer */
			
			counter++;				/* increments counter */
		} while (counter <= 30000);		/* till buffer is full */
	}
	
	/* checks and executes loop if data is to be stored in Sample2 buffer */
	if ( storeSampleValue == 2)
	{
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			DSPReadRX(&dataDSPSample2[counter]);
			/* reads 7 samples and discards, spurious data */
			
		/* Loop to transfer samples data from DSP RAM to Sample2 buffer */
		do
		{
			DSPReadRX(&dataDSPSample2[counter]);
			/* reads sampled data from DSP RAM and writes into Sample2 buffer */

			dataDSPSample2[ counter] =  (int ) (dataDSPSample2[ counter ] /2.40);
			/* scales data to range 0 to 2.5 volts */

			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints);  /* till buffer is full */
		
		/* loop to fill the remaining empty or previously used buffer with zeroes */
		do
		{
			dataDSPSample2[counter] = 0;
			/* writes zeroes to buffer */

			counter++;				/* increments counter */
		} while (counter <= 30000);		/* till buffer is full */
	}
			
	scrollValue = 0;					/* scroll feature is set to origin */
	DSPClose();						/* close the DSP and clean up the device */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- scrollLeft:sender

/* Invoked when the " < " button is clicked.  Used to scroll through the entire sampled data from
* ADC.  Scrolls left only.  If the waveform displayed starts from origin, the scroll left button is 
* inactive.  Sets a variable to -25.  This value is used by drawself method and subtracts this value
* from the starting location of  the sample buffer.  This produces a effect that the waveform is
*  moving left
*/

{
	if ( scrollValue >= 1 )				/* checks if left scrolling is possible */
	{
		scrollValue = scrollValue - 25;	/* if yes, sets  variable */
	} 
	else
	{
		scrollValue = 0;				/* if not, it does not scroll */
	}
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- scrollRight:sender

/* Invoked when the " > " button is clicked.  Used to scroll through the entire sampled data from
* ADC.  Scrolls right only.  If the waveform displayed is ending, the scroll left button is 
* inactive.  Sets a variable to +25.  This value is used by drawself method and adds this value
* from the starting location of  the sample buffer.  This produces a effect that the waveform is
*  moving right
*/

{
	if ( scrollValue <= 50000 )			/* checks if right scrolling is possible */
	{
		scrollValue = scrollValue + 25;	/* if yes, sets  variable */
	} 
	else
	{
		scrollValue = scrollValue;		/* if not, it does not scroll */
	}
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- scrollLeftMore:sender

/* Invoked when the " << " button is clicked.  Used to scroll through the entire sampled data from
* ADC, in bigger increments.  Scrolls left only.  If the waveform displayed starts from origin, the 
* scroll left button is inactive.  Sets a variable to -600.  This value is used by drawself method and 
* subtracts this value from the starting location of  the sample buffer.  This produces a effect that the 
* waveform is moving left in bigger increments than when used with "<" button
*/

{
	if ( scrollValue >= 1 )				/* checks if left scrolling is possible */
	{
		scrollValue = scrollValue - 600;	/* if yes, sets  variable */
	} 
	else
	{
		scrollValue = 0;				/* if not, it does not scroll */
	}

	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- scrollRightMore:sender

/* Invoked when the " >> " button is clicked.  Used to scroll through the entire sampled data from
* ADC, in bigger increments.  Scrolls right only.  If the waveform displayed is ending, the 
* scroll right  button is inactive.  Sets a variable to +600.  This value is used by drawself method and 
* adds this value from the starting location of  the sample buffer.  This produces a effect that the 
* waveform is moving right in bigger increments than when used with ">" button
*/

{
	if ( scrollValue <= 50000 )			/* checks if right scrolling is possible */
	{
		scrollValue = scrollValue + 600;	/* if yes, sets  variable */
	} 
	else
	{
		scrollValue = scrollValue;		/* if not, it does not scroll */
	}
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- scrollOrigin:sender

/* Invoked when the "  " button is clicked.  Used to display the samples data waveform from
* origin.  Sets a variable to 0.  This value is used by drawself method and takes this value as the
* starting location of  the sample buffer.
*/

{
	scrollValue = 0;					/* sets variable to origin */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- reproduce:sender

/* Invoked when Reproduce button is clicked .  Reproduces sampled data from ADC at X1, X5, 
* X10 and X100 times the frequency.  Reproduces the waveform stored in either Sample1 or
* Sample2 buffer, whichever is selected.  The waveform reproduced is either periodic or 
* non-periodic, depending on the selection.  Loads the DSP bootstrap program and transfers 
* the buffer data into DSP RAM.
*/

{
	int counter;
	waveSelected = 0;					/* disables any previously set wave for signal generator */
	counter = 0;						/* initialize counter */
	do {
			dataDSP[ counter ] = 0;	/* buffer is filled with zeroes, to erase previous data */
			counter++;				/* increment counter */
	     } while (counter <= 30000);		/* repeat till buffer is full */
	
	if ( periodicValue == 1)				
	{
	DSPBootFile("/LocalApps/SignalLab.app/dacReproduce.lod");
	/* Loads DSP bootstrap program to reproduce waveform periodically, if variable is set */

	} else
	{
	DSPBootFile("/LocalApps/SignalLab.app/dacReproduceOnce.lod");
	/* Loads DSP bootstrap program to reproduce waveform only once, if variable is not set */

	}
	DSPSetHF0();					/* sets host flag for communication with DSP */
	DSPSetHF1();					/* sets host flag for communication with DSP */
	DSPWriteTX(dacSelected);			/* transfers the selected DAC to DSP program */
	DSPWriteTX( maxSamplePoints);	/* transfers maximum samples data to DSP program */
	DSPWriteTX( xValue );	
	/* transfers the delay value which controls the reproducing frequency */

	counter =0;						/* initialize counter */
	
	/* Executes loop to transfer data to DSP RAM, contained in Sample1 buffer */
	if ( storeSampleValue == 1)
	{
		do
		{
			DSPWriteTX(dataDSPSample1[ counter ]);
			/* writes to DSP, the Sample1 buffer contents */

			counter++;					/* increment counter */
		} while (counter <= maxSamplePoints);   /* till all data is transfered */
	}
	
	/* Executes loop to transfer data to DSP RAM, contained in Sample2 buffer */
	if ( storeSampleValue == 2)
	{
		do
		{
			DSPWriteTX(dataDSPSample2[ counter ]);
			/* writes to DSP, the Sample2 buffer contents */

			counter++;				/* increment counter */
		} while (counter <= maxSamplePoints);  /* till all data is transfered */
	}

	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}




- storeSample1:sender

/* Invoked when Sample 1 button is clicked.  Selects Sample 1 as the buffer to store captured
* data
*/

{
	storeSampleValue = 1;				/* Sample 1 buffer is used to store captured data */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- storeSample2:sender

/* Invoked when Sample 2 button is clicked.  Selects Sample 2 as the buffer to store captured
* data
*/

{
	storeSampleValue = 2;				/* Sample 1 buffer is used to store captured data */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- outX1:sender

/* Invoked when 'x1' button is clicked.  Sets a variable to indicate the reproducing (X1) frequency
* of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses this
* information to reproduce waveform at the set frequency
*/

{
	xValue = 220;						/* X1  frequency */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- outX5:sender

/* Invoked when 'x5' button is clicked.  Sets a variable to indicate the reproducing (X5) frequency
* of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses this
* information to reproduce waveform at the set frequency
*/

{
	xValue = 45;						/* X5  frequency */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- outX10:sender

/* Invoked when 'x10' button is clicked.  Sets a variable to indicate the reproducing (X10) frequency
* of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses this
* information to reproduce waveform at the set frequency
*/

{
	xValue = 22;						/* X10  frequency */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- outX100:sender

/* Invoked when 'x100' button is clicked.  Sets a variable to indicate the reproducing (X100) 
* frequency of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses 
* this information to reproduce waveform at the set frequency
*/

{
	xValue = 2;						/* X100 frequency */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- periodic:sender

/* Invoked when 'Periodic' button is clicked.  Sets a variable to indicate continuous reproducing 
* waveform of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses 
* this information to reproduce continuous waveform
*/

{
	periodicValue = 1;					/* periodic waveform */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- nonPeriodic:sender

/* Invoked when 'Non-periodic' button is clicked.  Sets a variable to indicate a once only reproducing 
* waveform of captured data ( both Sample1 and Sample2 buffers ).  The method 'reproduce' uses 
* this information to reproduce waveform once
*/

{
	periodicValue = 2;					/* non-periodic waveform */
	[self display];					/* invokes drawself method */;
	return self;						/* self is the return value of this method */
}

- gainUp:sender

/* Invoked when ' + ' button is clicked.  Increases the voltage gain of selected sample buffer 
* (Sample1 or Sample2) by 10%.  drawself method is then executed which displays the
* changes to the sampled data on the "scope "
*/

{
	int counter;
	int atemp, btemp;					/* temporary variables */
	counter = 0;						/* counter initialization */
	
	/* if Sample1 is selected as data capture buffer, the gain is increased by 10% */
	if ( storeSampleValue == 1)
	{
		do
		{
			btemp = dataDSPSample1[ counter ]; /* stores data in temporary variable */
			atemp =  ( int) ( btemp *   1.1 ) ;	       /* increases by 10% */
			dataDSPSample1[ counter ] =  atemp; /* moves data back to buffer */
			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25); /* till all data gain is increased */
	}
	
	/* if Sample2 is selected as data capture buffer, the gain is increased by 10% */
	if ( storeSampleValue == 2)
	{
		do
		{
			btemp = dataDSPSample2[ counter ] ;  /* stores data in temporary variable */
			atemp =  ( int) ( btemp *   1.1 ) ;            /* increases by 10% */
			dataDSPSample2[ counter ] =  atemp;  /* moves data back to buffer */
			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25);  /* till all data gain is increased */

	}

	[self display];						/* invokes drawself method */;
	return self;							/* self is the return value of this method */
}

- gainDown:sender

/* Invoked when ' - ' button is clicked.  Decreases the voltage gain of selected sample buffer 
* (Sample1 or Sample2) by 10%.  drawself method is then executed which displays the
* changes to the sampled data on the "scope "
*/

{
	int counter;
	int atemp, btemp;					/* temporary variables */
	counter = 0;						/* counter initialization */
	
	/* if Sample1 is selected as data capture buffer, the gain is decreased by 10% */
	if ( storeSampleValue == 1)
	{
		do
		{
			btemp = dataDSPSample1[ counter ] ;   /* stores data in temporary variable */
			atemp =  ( int) ( btemp *   0.9 ) ;            /* decreases by 10% */
			dataDSPSample1[ counter ] =  atemp;  /* moves data back to buffer */
			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25);   /* till all data gain is decreased */
	}
	
	/* if Sample2 is selected as data capture buffer, the gain is decreased by 10% */
	if ( storeSampleValue == 2)
	{
		do
		{
			btemp = dataDSPSample2[ counter ] ;  /* stores data in temporary variable */
			atemp =  ( int) ( btemp *   0.9 ) ;             /* decreases by 10% */
			dataDSPSample2[ counter ] =  atemp;    /* moves data back to buffer */
			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25);   /* till all data gain is decreased */

	}

	[self display];						/* invokes drawself method */;
	return self;							/* self is the return value of this method */
}

- mix1Plus2:sender

/* Invoked when 'Sample 1+2 -> 1' button is clicked.  Averages Sample1 and Sample2 data 
* buffers, with the result stored in buffer Sample1.  Drawself method is then executed which 
* displays the contents of Sample1 buffer on the "scope"
*/

{
	int counter;
	int ctemp, dtemp, etemp;				/* temporary variables */
	counter = 0;							/* initialize counter */
		do
		{
			ctemp = dataDSPSample1[ counter ] ;   
			/* moves data of Sample1 buffer into temporary variable */

			dtemp = dataDSPSample2[ counter ] ;
			/* moves data of Sample2 buffer into temporary variable */

			etemp =  ( int) ( ( ctemp  + dtemp) / 2 ) ;
			/* averages data of both buffers */

			dataDSPSample1[ counter ] =  etemp;
			/* writes result into Sample1 buffer */

			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25);  /* till all data is averaged */

	[self display];						/* invokes drawself method */;
	return self;							/* self is the return value of this method */
}

- mix1Minus2:sender

/* Invoked when 'Sample 1-2 -> 1' button is clicked.  Subtracts Sample2 from Sample1 data 
* buffers, with the result stored in buffer Sample1.  Drawself method is then executed which 
* displays the contents of Sample1 buffer on the "scope"
*/

{
	int counter;
	int ctemp, dtemp, etemp;				/* temporary variables */
	counter = 0;							/* initialize counter */
		do
		{
			ctemp = dataDSPSample1[ counter ] ;
			/* moves data of Sample1 buffer into temporary variable */

			dtemp = dataDSPSample2[ counter ] ;
			/* moves data of Sample2 buffer into temporary variable */

			etemp =  ( int) ( ( ctemp  - dtemp)  ) ;
			dataDSPSample1[ counter ] =  etemp;
			/* subtracts data of  buffers */

			counter++;					/* increments counter */
		} while (counter <= maxSamplePoints + 25);   /* till all data is subtracted */

	[self display];						/* invokes drawself method */;
	return self;							/* self is the return value of this method */
}

@end
