#ifdef APOLLO
#include <libc.h>
#endif
#ifdef SUN
#include "SUN_libc.h"
#endif
#include "Macros.h"
#include "polynomial.h"
#include <string.h>
#include <stdlib.h>

main(int argc, char** argv)
{
    polynomial p, p1, p2;
    char* help_str;
                                    
    /* check for help switch and display help information */
    if (argc > 2) {
        cerr << "Too many arguments\n";
        cerr << "Usage: demovector [-h]\n";
        exit(0);
    }
    if (argc == 2) {
        if (!strcmp(argv[1],"-h")) {
            help_str = new (char *)[64];
            if (!help_str) {
                help_str = (char *)malloc(64);
            }
            strcpy(help_str,HELP_CMD);
            strcat(help_str," ");
            strcat(help_str,HELP_PATH);
            strcat(help_str,"poly");
            system(help_str);
            delete help_str;
        }
        else {
            cerr << "Usage: demovector [-h]\n";
            exit(0);
        }
    }

    cin >> p1 >> p2;
    cerr << "p1: " << p1 << endl;
    cerr << "p2: " << p2 << endl;

    p2.make_consistent();
    cerr << "p2: " << p2 << endl;

    cerr << "\nTest constructors:\n";
    polynomial pp(complex(3,4));
    cerr << "pp(complex(3,4)): " << pp << endl;

    cerr << "\nTest plus:\n";
    p = p1+p2;
    cerr << "p1+p2: " << p << endl;
    cerr << "and now call make_consistent():\n";
    p.make_consistent();
    cerr << "p1+p2: " << p << endl;

    cerr << "\nTest unary minus:\n";
    cerr << "-p1: " << -p1 << endl;
    cerr << "-p2: " << -p2 << endl;

    cerr << "\nTest subtraction:\n";
    p = p1-p2;
    cerr << "p1-p2: " << p << endl;
    cerr << "and now call make_consistent():\n";
    p.make_consistent();
    cerr << "p1-p2: " << p << endl;

    cerr << "\nTest multiplication:\n";
    p = p1*p2;
    cerr << "p1*p2: " << p << endl;

    cerr << "\nTest division:\n";
    transform_variable z;
    p1 = z^2;
    //causes a compilation warning : non-const temporary used
    p2 = z-1;
    cerr << "p2: " << p2 << endl;
    p2=p2*p2;
    cerr << "p1: " << p1 << endl;
    cerr << "p2: " << p2 << endl;
    p = divide( p1, p2 );
    cerr << "p1/p2: " << p << " with remainder "<< p1 << endl;

//  cerr << "\n now try setting a polynomial with a transform_variable:\n";
    /*
    p = (z^3) - (z^2) + 1;	// doesn't do int->double->complex->polynomial
    p = (z^3) - (z^2) + complex(1); // complains:
        "warning: temporary used for non-const polynomial & argument"
    */
//   p = (z^3) - (z^2);
//  cerr <<  "(z^3) - (z^2): " << p << endl;

    cerr << "\n now try the invert_argument() function:\n";
    p = (z^2);
    cerr <<  "p: " << p << endl;
    cerr <<  "p(z^-1): " << invert_argument(p) << endl;
    p.make_consistent();
    cerr <<  "p: " << p << endl;
    cerr <<  "p(z^-1): " << invert_argument(p) << endl;
    cin >> p;
    cerr <<  "p: " << p << endl;
    cerr <<  "p(z^-1): " << invert_argument(p) << endl;

    cerr << "\n now try the add_zero_roots() method:\n";
    p = p1;
    cerr <<  "p: " << p << endl;
    p.add_zero_roots( 3 );
    cerr <<  "p with 3 more zero roots: " << p << endl;
    p = p1;
    p.make_consistent();
    cerr <<  "p: " << p << endl;
    p.add_zero_roots( 3 );
    cerr <<  "p with 3 more zero roots: " << p << endl;

    cerr << "\n now try the integral() function:\n";
    cerr <<  "p: " << p1 << endl;
    cerr <<  "integral(p): " << integral(p1) << endl;

    cerr << "\n now try past failures:\n";
    cin >> p1;
    cerr <<  "p1: " << p1 << endl;
    p1.make_consistent();
    cerr <<  "p1: " << p1 << endl;

    cerr << "\n now try the multiplicity method:\n";
    cin >> p1;
    cerr << "p1= " << p1 << endl;
    p1.bless_roots(); //ADDED BY GIRI
    /* Got to calculate roots here as they are not yet found. Hence, rtn 
     * bless_roots is called. Gives the correct answer now.
     */
    cerr << "p1= " << p1 << endl;
    for( int i=0; i<p1.degree(); ++i )
	cerr << p1.root[i] << " has multiplicity "<<p1.multiplicity(i)<<endl;
    cerr <<  "now p1= " << p1 << endl;
}
