#define LBRACE '{'
#define RBRACE '}'

#include "vector.h"

vector::vector( const vector& a) { // a "copy constructor"
    register i = a.n; 
    n = i; 
    // Don't worry about v pointing to space previously allocated with new.
    v = new double[i]; 
    register double* thisv = v; 
    register double* av = a.v; 
    while (i--)
	*thisv++ = *av++; 
    }

vector& vector::operator=( const vector& a) { 
    register i = a.n; 
    if (i != n) {
	delete[] v;
	v = new double[n=i];
	}
    register double* thisv = v; 
    register double* av = a.v; 
    while (i--) 
	*thisv++ = *av++; 
    return *this; 
    } 

void   vector::set_size(int s) { 
    if(n==s)
	return;
    delete[] v; 
    v = new double[n=s]; 
    }

void   vector::change_size(int s) { 
    if(n==s)
	return;
    double* newv = new double[s]; 
    register i = s<n?s:n; 
    register double* thisv = v; 
    register double* nv = newv; 
    while (i--) 
	*nv++ = *thisv++; 
    delete[] v; 
    v = newv; 
    n = s; 
    } 

void   vector::initialize() { 
    register i = size(); 
    register double* thisv = v; 
    while (i--) 
	*thisv++ = 0; 
    } 

void	vector::set_small_to_zero( double tol ){ // set small elements to zero
    register double *thisv=v;
    for( int i=0; i<size(); ++i, ++thisv )
	if( fabs( *thisv ) < tol )
	    *thisv = 0;
    }

ostream& operator<<(ostream& s, const vector& z){
    s << LBRACE << ' ';
    for( int i=0; i<z.size(); ++i )
	s << z[i] << " ";
    return s << RBRACE;
    }

istream& operator>>(istream& s, vector& v) {
    double z;
    char c=0;

    s>>c;
    if (c == LBRACE) {
	int i=0;
	while( s>>z ) {
	    if( i >= v.size() )
		v.change_size( i+10 );
	    v[i++] = z;
	    s>>c;
	    if( c==RBRACE)
		break;
	    else
		s.putback(c);
        }
	v.change_size(i);
    }
    else {		// missing LBRACE
	/* set the vector to the null vector */
	/* ? call vector::~vector() ? */
	s.clear(ios::badbit); // what does this do?
    }
    return s;
}

cxVector::cxVector( const cxVector& a) { // a "copy constructor"
    register i = a.n; 
    n = i; 
    v = new complex[i]; 
    register complex* thisv = v; 
    register complex* av = a.v; 
    while (i--) *thisv++ = *av++; 
    }   

cxVector::cxVector(cxVector &a, cxVector &b) { 
    v = new complex[ n = a.n + b.n ]; 
    register complex *vv=v;
    for( int i=0; i<a.n; ++i )
	*vv++ = a[i];
    for( i=0; i<b.n; ++i )
	*vv++ = b[i];
    }   

cxVector& cxVector::operator=( const cxVector& a) { 
    register i = a.n; 
    if (i != n) {
	delete[] v;
	v = new complex[n=i];
	}
    register complex* thisv = v; 
    register complex* av = a.v; 
    while (i--) 
	*thisv++ = *av++; 
    return *this; 
    } 

void   cxVector::set_size(int s) { 
    if(n==s)
	return;
    delete[] v; 
    v = new complex[n=s]; 
    }

void   cxVector::change_size(int s) { 
    if(n==s)
	return;
    complex* newv = new complex[s]; 
    register i = s<n?s:n; 
    register complex* thisv = v; 
    register complex* nv = newv; 
    while (i--) 
	*nv++ = *thisv++; 
    delete[] v; 
    v = newv; 
    n = s; 
    } 

void   cxVector::reverse() { 	// reverse the order of the elements
    register complex *v1=&v[0], *v2=&v[n-1];
    while( v1 < v2 ){
	complex tmp=*v1;
	*v1++ = *v2;
	*v2-- = tmp;
	}
    }

void	cxVector::set_small_to_zero( double tol ){ // set small elements to zero
    for( int i=0; i<size(); ++i ){
	if( fabs( v[i].re ) < tol )
	    v[i].re = 0;
	if( fabs( v[i].im ) < tol )
	    v[i].im = 0;
	}
    }

ostream& operator<<(ostream& s, const cxVector& z){
    s << LBRACE << ' ';
    for( int i=0; i<z.size(); ++i )
	s << z[i] << " ";
    return s << RBRACE;
    }

istream& operator>>(istream& s, cxVector& v) {
    complex z;
    char c=0;

    s>>c;
    if (c == LBRACE) {
	int i=0;
	while( s>>z ){
	    if( i >= v.size() )
		v.change_size( i+10 );
	    v[i++] = z;
	    s>>c;
	    if( c==RBRACE)
		break;
	    else
		s.putback(c);
	    }
	v.change_size(i);
	}
    else {		// missing LBRACE
	/* set the vector to the null vector */
	/* ? call cxVector::~cxVector() ? */
	s.clear(ios::badbit); // what does this do?
	}

    return s;
    }
