#include "CofR.h"
#include "lu.h"
#include "stdlib.h"	// For the rand() function.

CofR::CofR( const CofR& a ){ // a "copy constructor"
    cerr << "Now here.\n";
    register i = a.ncoeffs(); 
    order = a.order; 
    coeffs = new double[i]; 
    register double* thisv = coeffs; 
    register double* av = a.coeffs; 
    while (i--)
	*thisv++ = *av++; 
    cerr << "Done copying.\n";
    }   

void CofR::set_order( int n ){
    if( order == n )
	return;
    delete coeffs;
    order = n;
    coeffs = new double[ncoeffs()];
    }

// Find coefficient values for the cascade-of-resonators structure.
// See pg 92 in Notebook 3.
CofR::CofR( const ratfn& H ){ 
    set_order( H.den.degree() );
    int i, i2, n = ncoeffs();
    double *beta = coeffs, *gamma = coeffs + order;

    transform_variable z;
    ratfn H_ = (z*(H.num-H.den))/H.num;

    polynomial zm1; zm1.set_coeffs(cxVector(-1,1));
    polynomial *r; r = new polynomial[order/2];
    /* ORDER THE RESONATORS IN THE ORDER OF THE NUMERATOR ROOTS */
    for( i=0; i<order/2; i++ ){
        cxVector roots(H.num.root[2*i],H.num.root[2*i+1]);
        r[i].set_roots(1,roots);
        gamma[i] = 2*(1-roots[0].re);
        }

    ratfn *g; g = new ratfn[order+1];
    g[order]=1;
    for( i=order-1, i2=order/2-1 ; i>=0; i-=2, i2-- ){
        g[i-1] = (z*g[i+1].num)/(r[i2]*g[i+1].den);
        g[i] = zm1*g[i-1];
        }
    delete r;

    // Compute the matrix equation
    double **a = new double*[order];
    for( i=0; i<order; ++i )
        a[i] = new double[order];
    // Pick a bunch of points in the z-plane
    for( i=0; i<order; ++i ){
        complex zz = 0.7634*exp(3.14*(i+0.4312)/order*complex(0,1));
        beta[i]=H_(zz).re;
        for( int j=0; j<order; ++j )
            a[i][j] = g[j](zz).re;
        }
    delete g;

    // Solve the matrix equation
    int *indx = new int[order];
    ludcmp( a, order, indx );
    lubksb( a, order, indx, beta );

    for( i=0; i<order; ++i )
        delete a[i];
    delete a;
    delete indx;

    // Set small coefficients to zero.
    for( i=0; i<n; ++i )
        if( fabs(coeffs[i]) < 1e-10 )
            coeffs[i] = 0;
    }

CofR& CofR::operator=( const CofR& a){ 
    register i = a.ncoeffs(); 
    set_order( a.order );
    register double* thisv = coeffs; 
    register double* av = a.coeffs; 
    while (i--) 
	*thisv++ = *av++; 
    return *this; 
    } 

void CofR::perturb( double amount, CofR &changed ){
    changed.set_order(order);
    double offset = 1 - amount;
    double randScale = amount/0x40000000;
    double *c1=changed.coeffs, *c0=coeffs;
    for( int i=ncoeffs(); i>0; --i )
	*c1++ = *c0++ * ( offset + randScale*rand() );
    }

ratfn CofR::NTF(){ 
    double *beta = coeffs, *gamma = coeffs + order;
    polynomial num=0;

    polynomial r, s, rootProduct=1;
    for( int i=0, i2=0; i<order; i+=2, i2++ ){
	double re = 1-gamma[i2]/2, im = sqrt(1-re*re);
        r.set_roots( 1, cxVector( complex(re,im), complex(re,-im) ) );
	s.set_coeffs( cxVector( beta[i]-beta[i+1], beta[i+1] ) );
	s.bless_roots();
	s.add_zero_roots( order/2 -1 -i2 );
	num += s * rootProduct;
	rootProduct = rootProduct * r;
        }

    ratfn H;
    H.num = rootProduct;
    H.den = rootProduct - num;
    H.den.bless_roots();
    return H;
    }

ostream& operator<<(ostream& s, const CofR& a){
    double *c=a.coeffs;
    s << "order= " << a.order << '\n';
    s << "beta= ";
    for( int i=0; i<a.order; ++i )
        s << *c++ << ' ';
    s << '\n';
    s << "gamma= ";
    for( ; i<a.ncoeffs(); ++i )
        s << *c++ << ' ';
    s << '\n';

    return s;
    }

istream& operator>>(istream& s, CofR& a) {
    // Not implemented yet.
    int order;
    char str[80];
    s >> str >> order;
    //if( strcmp
    if( a.order != order ){
	delete a.coeffs;
	a.order = order;
	a.coeffs = new double[a.ncoeffs()];
	}
    double *c = a.coeffs;
    s >> str;
    //if( strcmp
    for( int i=0; i<order; ++i )
        s >> *c++;
    s >> str;
    //if( strcmp
    for(; i<a.ncoeffs(); ++i )
        s >> *c++;

    return s;
    }
