#include "lu.h"
#include <libc.h>
#include <iostream.h>
#include <math.h>

/* Adapted from Numerical Recipes in C, pg 43 */

int ludcmp( double **a, int n, int *indx ){
    int imax, i, j, k;
    int parity=1;
    double *vv = new double[n];		// stores the scaling for each row.

    for( i=0; i<n; ++i ){
	double big = fabs(a[i][0]);
	for( j=1; j<n; ++j )
	    big = max( big, fabs(a[i][j]) );
	if( big==0 ){ cerr << "Singular matrix.\n"; exit(-1); }
	vv[i] = 1/big;
	}

    for( j=0; j<n; ++j ){	// Loop over the columns, as per Crout's method
	for( i=0; i<j; ++i ){
	    double sum=a[i][j];
	    for( k=0; k<i; ++k )
		sum -= a[i][k]*a[k][j];
	    a[i][j] = sum;
	    }

	double big=0;		// Search for the largest pivot
	for( i=j; i<n; ++i ){
	    double sum=a[i][j], temp;
	    for( k=0; k<j; ++k )
		sum -= a[i][k]*a[k][j];
	    a[i][j] = sum;
	    if( (temp = vv[i]*fabs(sum)) >= big ){
		big = temp;
		imax = i;
		}
	    }
	if( j!=imax ) {
	    // Interchange rows
	    double *temp=a[imax];
	    a[imax] = a[j];
	    a[j] = temp;
	    vv[imax] = vv[j];
	    parity = -parity;
	    }
	indx[j] = imax;
	if( a[j][j] == 0.0 ) a[j][j] = 100*macheps();
	if( j < n-1 ){
	    double scale = 1/a[j][j];
	    for( i=j+1; i<n; ++i ) 
		a[i][j] *= scale;
	    }
	}

    delete vv;
    return parity;
    }

void lubksb( double **a, int n, int *indx, double *b){
    int ii=-1;

    for( int i=0; i<n; ++i ){
	int ip=indx[i];
	double sum=b[ip];
	b[ip]=b[i];
	if( ii != -1 )
	    for( int j=ii; j<i; ++j )
		sum -= a[i][j]*b[j];
	else if(sum) 
	    ii = i;
	b[i] = sum;
	}
    for( i=n-1; i>=0; i-- ){
	double sum=b[i];
	for( int j=i+1; j<n; ++j )
	    sum -= a[i][j]*b[j];
	b[i] = sum/a[i][i];
	}
    }
