#include <libc.h>
#include <fstream.h>
#include "ratfn.h"

/*
 * Calculate the in-band noise power, given H.
 * (Assumes e is white and uniformly distributed over [-1,+1]).
 * Usage: noisePower [-opts] fn f1 f2
 * options are:
 *	d	print the power in dB
 *	D	set the internal 'debug' flag
 * 
 * arguments are:
 *	fn	a file containing the error transfer function.
 *	f1	the lower frequency limit, relative to pi.
 *	f2	the upper frequency limit, relative to pi. 0 <= f1 < f2 <= 1
 * 
 */


char *CmdName, *modulator;
double f1, f2, df;
int indB=0, debug=0;

void parse_args( int argc, char *argv[] ) {
    CmdName = *argv;
    while( --argc > 0 && **++argv == '-' ){
	char c;
	while( c=*++(*argv) )
	    switch( c ){
		case 'd':
		    indB = 1;
		    break;
		case 'D':
		    debug = 1;
		    break;
		default:
		    cerr << CmdName <<": unknown option "<< c << endl;
		    exit(-1);
		}
	}
    if( argc != 3 ){
	switch( argc ){
	case 0:
	    cerr << CmdName <<": name of the file containing a definition of the error tf missing.\n";
	case 1:
	    cerr << CmdName <<": lower frequency limit missing.\n";
	case 2:
	    cerr << CmdName <<": upper frequency limit missing.\n";
	    break;
	default:
	    cerr << CmdName <<": too many parameters " << argv[3] <<" etc.\n";
	    }
	exit(-1);
	}
    modulator = argv[0];
    f1 = M_PI*atof( argv[1] );
    f2 = M_PI*atof( argv[2] );
    if( f1 < 0 ){
	cerr << CmdName <<": f1 is negative.\n";
	exit(-1);
	}
    if( f1 > f2 ){
	cerr << CmdName <<": f1 is bigger than f2.\n";
	exit(-1);
	}
    if( f2 > M_PI ){
	cerr << CmdName <<": f2 is bigger than pi.\n";
	exit(-1);
	}
    df = f2-f1;
    }

main(int argc, char *argv[]){
    parse_args( argc, argv );
    ratfn h;
    if( !h.read_in( modulator ) )
	exit(-1);
    double N2 = h.integrate_mag2_along_unit_circle(f1,f2) / (3*M_PI);
    if( indB )
	cout << 10*log10(N2) << endl;
    else
	cout << N2 << endl;
    exit(0);
    }
