#include <stdlib.h>
#include <libc.h>
#include <fstream.h>
#include "mainhdr.h"
#include "CofR.h"

double noisePower(ratfn &H, double w1, double w2 ){
    return 10*log10( H.integrate_mag2_along_unit_circle(w1,w2) / (3*M_PI) );
    }

int compareDoubles( const void *x, const void *y ){ 
    double diff = *(double *)x - *(double *)y;
    if(diff>0) return 1;
    else if(diff<0) return -1;
    else return 0;
    }

// Global variables that make life easier.
ratfn H;
double w0, R;
int order, order2;

#include "parseFile.C"

main(int argc, char *argv[]){
    double tol=.01, pntrials=100;
    int all=0;
    // This would be cleaner if C++ allowed array initialization.
    parameterStruct parms[4], *p=parms;
    p->string = "trials";	p++->parameter = &pntrials;
    p->string = "tol";		p++->parameter = &tol;
    p->string = (char *)NULL;	p++->parameter = (double *)NULL;
    flagStruct flags[3], *f=flags;
    f->string = "debug";      	f++->flag = &debug;
    f->string = "all";      	f++->flag = &all;
    f->string = (char *)NULL;	f++->flag = 0;
    parseArgs( argc, argv, parms, flags );

    if( argc != 1 ){
        cerr << "A file containing a modulator description is needed.\n";
        exit(-1);
        }
    if( !parseFile( argv[1] ) ) exit(-1);
    order = H.den.degree();
    order2 = order/2;
    w0 *= M_PI;
    double dw=M_PI/R, w1=0, w2=w0+dw;
    if(w0!=0){
	dw *= 0.5;
	w1 = w0-dw;
	w2 = w0+dw;
	}

    CofR realization( H );
    if( debug ){
	cerr << "Ideal realization:\n" << realization << '\n';
	cerr << "NTF as determined from the realization:\n"<< realization.NTF()<<'\n';
	}
    DEBUGSHOW(w1)
    DEBUGSHOW(w2)

    const int ntrials = (int)pntrials;
    double *result = new double[ntrials];

    CofR perturbed;
    for( int trial=0; trial < ntrials; ++trial ){
	realization.perturb( tol, perturbed );
	H = perturbed.NTF();
	result[trial] = noisePower(H,w1,w2);
	if( debug ) 
	    cerr << trial << ' ' << result[trial] << '\n';
	}

    cout.precision(5);
    qsort(result, trial, sizeof(double), compareDoubles);
    if(all) // Print the sorted list of results.
	for( trial=0; trial < ntrials; ++trial )
	    cout << result[trial] << '\n';

    // Print the 95th percentile and the maximum noise power, in dB
    cout << tol << ' ' << result[(int)(ntrials*0.95)-1] 
	 << ' ' << result[ntrials-1] << " \n";
    delete result;
    }
