/* noise.c
 *  a noise source
 *  USAGE: noise [options] N=duration(1024) [rms=power | A=amplitude(1)]
 *  options:
 *    -r  random: generate a different sequence with each call.
 *    -gaussian  gaussian
 *    -debug  debug
 */

#include <sys/time.h>
#include <math.h>
#include "mainhdr.h"

double scale = 2./0x80000000;

// Return a zero-mean, unit-variance gaussian-distributed random number.
double gaussian_rv(){ // Adapted from "Numerical Recipes in C", page 217.
    static int haveASpare=0;
    static double spare;
    if( haveASpare ){ // Return the spare deviate from a previous call.
	haveASpare=0;
	return spare;
	}
    double r, v1, v2;
    do{ // Pick two random numbers, uniformly distributed in the square [-1,+1]^2.
       v1 = scale*random()-1;
       v2 = scale*random()-1;
       r = v1*v1 + v2*v2;
       } while(r>=1); // Only accept those inside the unit circle.
    r = sqrt(-2*log(r)/r);
    spare = r*v2;
    haveASpare=1;
    return r*v1;
    }

main( int argc, char *argv[] ){
    double N_d=1024;
    double rms=-1, amplitude=-1;
    int set_random=0, gaussian=0;

    // Parse the arguments.
    struct parameterStruct parameters[4], *p=parameters;
    p->string = "N";		p++->parameter = &N_d;
    p->string = "rms";		p++->parameter = &rms;
    p->string = "A";		p++->parameter = &amplitude;
    p->string = (char*)NULL;	p++->parameter = (double *)NULL;
    // Flags:
    struct flagStruct flags[4], *f=flags;
    f->string = "gaussian";    	f++->flag = &gaussian;
    f->string = "r";        	f++->flag = &set_random;
    f->string = "debug";       	f++->flag = &debug;
    f->string = (char*)NULL;    f++->flag = 0;
    parseArgs( argc, argv, parameters, flags );
    if( argc !=0 ) {
	ERRORMESG("Arguments which were not parsed:");
	while( argc-- )
	    cerr << (*argv++) << ' ';
	cerr << endl;
	exit(-1);
	}
    long N = (long)N_d;

    double A;
    if( rms==-1 ){
	if( amplitude==-1 )
	    amplitude=1;
	A = amplitude;
	rms = A/sqrt(3);
	}
    else if( amplitude==-1 )
	A = sqrt(3)*rms;
    else {
	ERRORMESG("Please supply either a rms= OR a A= argument.");
	return -1;
	}

    if( set_random ){ // use a seed derived from the current time.
	struct timeval t;
	gettimeofday( &t, 0 );
	srandom( (int) (t.tv_sec + t.tv_usec) );
	}

    if(gaussian){
	DEBUGSHOW(rms);
	while( N-- >0 )
	    cout << rms*gaussian_rv() << endl;
	}
    else{ // Uniform distribution
	scale *= A;
	while( N-- >0 )
	    cout << scale*random() - A  << endl;
	}
    return 1;
    }
