FileStream subclass: #LineShape
	instanceVariableNames: 'size maxHeight widthRatio width filename comment widthComment symetry '
	classVariableNames: 'Antisymetric Symetric '
	poolDictionaries: ''
	category: 'LineShapes for FDC-ESR'!
LineShape comment:
'This is an abstract class. A new class must be derived . See Lorentzian.'!


!LineShape methodsFor: 'Accessing'!

size: aNumber 
	size _ aNumber.
	width _ size / widthRatio! !

!LineShape methodsFor: 'Calculating'!

calculate
	self subclassResponsibility! !

!LineShape methodsFor: 'Writing'!

execute
	self writeHeader; calculate; close.
	Transcript cr; tab; show: ' finished'; cr!

writeData: data 
	| newLine aString |
	newLine _ '
'.
	(data isKindOf: String)
		ifTrue: [aString _ data]
		ifFalse: [aString _ data printString].
	self nextPutAll: aString , newLine
	"Transcript show: aString; cr"
	"for debugging"!

writeHeader
	Transcript clear; cr; show: comment , ' lineshape ASCII file generator'; cr; tab; show: 'for the FDC-ESR program'; cr; cr; tab; show: 'Please wait'.
	" <1>  write name of the lineshape ( string of maximun N characters ). 
	  
	 It will be displayed in FDC-ESR while loading the data from disk"
	self writeData: comment.
	" <2> write comment for the linewidth ( string of maximun N  
	 
	characters). It will be displayed in FDC-ESR when entering the line  
	 
	width"
	self writeData: widthComment.
	" <3> write total number of element in the lineshape"
	self writeData: size.
	" <4> write width of the lineshape ( corresponding to comment at <2>  
	 
	)"
	self writeData: width asInteger.
	" <5> write symmetry of the lineshape"
	self writeData: symetry.
	" <6> write the largest unsigned integer value for the digital lineshape 
	  
	, all data values (see <7> )will be scaled and rounded to a 0 - 
	maxHeight  
	 
	range by the COMPILER program  
	 
	low : decrease precision but decrease overflow risk  
	 
	high : increase precision but increase overflow risk 
	 
	"
	0 <= maxHeight & (maxHeight <= 65535) ifFalse: [self error: 'maxHeight is not correct'].
	self writeData: maxHeight! !

!LineShape methodsFor: 'initialization'!

initialize
	"set default values"

	size _ 32000.
	widthRatio _ 16.
	maxHeight _ 32767.
	"May not be larger than 65535"
	width _ size / widthRatio! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LineShape class
	instanceVariableNames: ''!


!LineShape class methodsFor: 'new instance'!

newFileNamed: aFileDesignator 

	^(super newFileNamed: aFileDesignator ) text initialize!

newFileNamed: shapeFileName size: aNumber 
	| aLineShape |
	aLineShape _ self newFileNamed: shapeFileName.
	aLineShape size: aNumber.
	^aLineShape! !

!LineShape class methodsFor: 'class initialization'!

initialize
	"LineShape initialize"

	Symetric _ #SYMETRIC.
	Antisymetric _ #ANTISYMETRIC! !

LineShape initialize!


LineShape subclass: #Lorentzian
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LineShapes for FDC-ESR'!
Lorentzian comment:
'(Lorentzian newFileNamed: ''lineshap.asc'' size: 1600 ) execute 

(Lorentzian newFileNamed: ''lineshap.asc''  ) execute '!


!Lorentzian methodsFor: 'Calculating'!

calculate
	" lorentzian function : Y = A*W*W / (W*W + 4*(X0-X)*(X0-X) )"
	" absolute height is not important ( A*W*W is a constant )    
	 
	=>  Y = constant / (W*W + 4*(X0-X)*(X0-X) )"

	| X0 constant W |
	W _ width.
	X0 _ size / 2.
	constant _ 100000.0.
	(0 to: size)
		do: [:X | self writeData: (constant / (W * W + (4 * (X0 - X) * (X0 - X)))) asFloat]! !

!Lorentzian methodsFor: 'Initialization'!

initialize
	"put here everything specific to the lineshape"

	super initialize.
	comment _ 'Lorentzian'.
	widthComment _ 'half height linewidth'.
	symetry _ Symetric! !